// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "chrome/browser/apps/foundation/app_service/public/mojom/app_registry.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "chrome/browser/apps/foundation/app_service/public/mojom/app_registry.mojom-shared-message-ids.h"
namespace apps {
namespace mojom {
const char AppRegistry::Name_[] = "apps.mojom.AppRegistry";

class AppRegistry_GetApps_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  AppRegistry_GetApps_ForwardToCallback(
      AppRegistry::GetAppsCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  AppRegistry::GetAppsCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(AppRegistry_GetApps_ForwardToCallback);
};

AppRegistryProxy::AppRegistryProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void AppRegistryProxy::GetApps(
    GetAppsCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "apps::mojom::AppRegistry::GetApps");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kAppRegistry_GetApps_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::apps::mojom::internal::AppRegistry_GetApps_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(AppRegistry::Name_);
  message.set_method_name("GetApps");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new AppRegistry_GetApps_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void AppRegistryProxy::SetAppPreferred(
    const std::string& in_app_id, ::apps::mojom::AppPreferred in_state) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "apps::mojom::AppRegistry::SetAppPreferred");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kAppRegistry_SetAppPreferred_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::apps::mojom::internal::AppRegistry_SetAppPreferred_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->app_id)::BaseType::BufferWriter
      app_id_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_app_id, buffer, &app_id_writer, &serialization_context);
  params->app_id.Set(
      app_id_writer.is_null() ? nullptr : app_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->app_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null app_id in AppRegistry.SetAppPreferred request");
  mojo::internal::Serialize<::apps::mojom::AppPreferred>(
      in_state, &params->state);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(AppRegistry::Name_);
  message.set_method_name("SetAppPreferred");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class AppRegistry_GetApps_ProxyToResponder {
 public:
  static AppRegistry::GetAppsCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<AppRegistry_GetApps_ProxyToResponder> proxy(
        new AppRegistry_GetApps_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&AppRegistry_GetApps_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~AppRegistry_GetApps_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  AppRegistry_GetApps_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "AppRegistry::GetAppsCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      std::vector<::apps::mojom::AppInfoPtr> in_app_infos);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(AppRegistry_GetApps_ProxyToResponder);
};

bool AppRegistry_GetApps_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "apps::mojom::AppRegistry::GetAppsCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::AppRegistry_GetApps_ResponseParams_Data* params =
      reinterpret_cast<
          internal::AppRegistry_GetApps_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  std::vector<::apps::mojom::AppInfoPtr> p_app_infos{};
  AppRegistry_GetApps_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadAppInfos(&p_app_infos))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "AppRegistry::GetApps response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_app_infos));
  return true;
}

void AppRegistry_GetApps_ProxyToResponder::Run(
    std::vector<::apps::mojom::AppInfoPtr> in_app_infos) {
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kAppRegistry_GetApps_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::apps::mojom::internal::AppRegistry_GetApps_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->app_infos)::BaseType::BufferWriter
      app_infos_writer;
  const mojo::internal::ContainerValidateParams app_infos_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::apps::mojom::AppInfoDataView>>(
      in_app_infos, buffer, &app_infos_writer, &app_infos_validate_params,
      &serialization_context);
  params->app_infos.Set(
      app_infos_writer.is_null() ? nullptr : app_infos_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->app_infos.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null app_infos in ");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)apps::mojom::AppRegistry::GetAppsCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(AppRegistry::Name_);
  message.set_method_name("GetApps");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool AppRegistryStubDispatch::Accept(
    AppRegistry* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kAppRegistry_GetApps_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)apps::mojom::AppRegistry::GetApps",
               "message", message->name());
#endif
      break;
    }
    case internal::kAppRegistry_SetAppPreferred_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)apps::mojom::AppRegistry::SetAppPreferred",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::AppRegistry_SetAppPreferred_Params_Data* params =
          reinterpret_cast<internal::AppRegistry_SetAppPreferred_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_app_id{};
      ::apps::mojom::AppPreferred p_state{};
      AppRegistry_SetAppPreferred_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadAppId(&p_app_id))
        success = false;
      if (!input_data_view.ReadState(&p_state))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "AppRegistry::SetAppPreferred deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetAppPreferred(
std::move(p_app_id), 
std::move(p_state));
      return true;
    }
  }
  return false;
}

// static
bool AppRegistryStubDispatch::AcceptWithResponder(
    AppRegistry* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kAppRegistry_GetApps_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)apps::mojom::AppRegistry::GetApps",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      internal::AppRegistry_GetApps_Params_Data* params =
          reinterpret_cast<
              internal::AppRegistry_GetApps_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      AppRegistry_GetApps_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "AppRegistry::GetApps deserializer");
        return false;
      }
      AppRegistry::GetAppsCallback callback =
          AppRegistry_GetApps_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetApps(std::move(callback));
      return true;
    }
    case internal::kAppRegistry_SetAppPreferred_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)apps::mojom::AppRegistry::SetAppPreferred",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool AppRegistryRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "AppRegistry RequestValidator");

  switch (message->header()->name) {
    case internal::kAppRegistry_GetApps_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::AppRegistry_GetApps_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kAppRegistry_SetAppPreferred_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::AppRegistry_SetAppPreferred_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool AppRegistryResponseValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "AppRegistry ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kAppRegistry_GetApps_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::AppRegistry_GetApps_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
void AppRegistryInterceptorForTesting::GetApps(GetAppsCallback callback) {
  GetForwardingInterface()->GetApps(std::move(callback));
}
void AppRegistryInterceptorForTesting::SetAppPreferred(const std::string& app_id, ::apps::mojom::AppPreferred state) {
  GetForwardingInterface()->SetAppPreferred(std::move(app_id), std::move(state));
}
AppRegistryAsyncWaiter::AppRegistryAsyncWaiter(
    AppRegistry* proxy) : proxy_(proxy) {}

AppRegistryAsyncWaiter::~AppRegistryAsyncWaiter() = default;

void AppRegistryAsyncWaiter::GetApps(
    std::vector<::apps::mojom::AppInfoPtr>* out_app_infos) {
  base::RunLoop loop;
  proxy_->GetApps(
      base::BindOnce(
          [](base::RunLoop* loop,
             std::vector<::apps::mojom::AppInfoPtr>* out_app_infos
,
             std::vector<::apps::mojom::AppInfoPtr> app_infos) {*out_app_infos = std::move(app_infos);
            loop->Quit();
          },
          &loop,
          out_app_infos));
  loop.Run();
}

}  // namespace mojom
}  // namespace apps

namespace mojo {

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif