// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_BROWSER_APPS_FOUNDATION_APP_SERVICE_PUBLIC_MOJOM_APP_REGISTRY_MOJOM_H_
#define CHROME_BROWSER_APPS_FOUNDATION_APP_SERVICE_PUBLIC_MOJOM_APP_REGISTRY_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "chrome/browser/apps/foundation/app_service/public/mojom/app_registry.mojom-shared.h"
#include "chrome/browser/apps/foundation/app_service/public/mojom/types.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace apps {
namespace mojom {
class AppRegistry;
using AppRegistryPtr = mojo::InterfacePtr<AppRegistry>;
using AppRegistryPtrInfo = mojo::InterfacePtrInfo<AppRegistry>;
using ThreadSafeAppRegistryPtr =
    mojo::ThreadSafeInterfacePtr<AppRegistry>;
using AppRegistryRequest = mojo::InterfaceRequest<AppRegistry>;
using AppRegistryAssociatedPtr =
    mojo::AssociatedInterfacePtr<AppRegistry>;
using ThreadSafeAppRegistryAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<AppRegistry>;
using AppRegistryAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<AppRegistry>;
using AppRegistryAssociatedRequest =
    mojo::AssociatedInterfaceRequest<AppRegistry>;


class AppRegistryProxy;

template <typename ImplRefTraits>
class AppRegistryStub;

class AppRegistryRequestValidator;
class AppRegistryResponseValidator;

class  AppRegistry
    : public AppRegistryInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = AppRegistryProxy;

  template <typename ImplRefTraits>
  using Stub_ = AppRegistryStub<ImplRefTraits>;

  using RequestValidator_ = AppRegistryRequestValidator;
  using ResponseValidator_ = AppRegistryResponseValidator;
  enum MethodMinVersions : uint32_t {
    kGetAppsMinVersion = 0,
    kSetAppPreferredMinVersion = 0,
  };
  virtual ~AppRegistry() {}


  using GetAppsCallback = base::OnceCallback<void(std::vector<::apps::mojom::AppInfoPtr>)>;
  virtual void GetApps(GetAppsCallback callback) = 0;

  virtual void SetAppPreferred(const std::string& app_id, ::apps::mojom::AppPreferred state) = 0;
};
class  AppRegistryInterceptorForTesting : public AppRegistry {
  virtual AppRegistry* GetForwardingInterface() = 0;
  void GetApps(GetAppsCallback callback) override;
  void SetAppPreferred(const std::string& app_id, ::apps::mojom::AppPreferred state) override;
};
class  AppRegistryAsyncWaiter {
 public:
  explicit AppRegistryAsyncWaiter(AppRegistry* proxy);
  ~AppRegistryAsyncWaiter();
  void GetApps(
      std::vector<::apps::mojom::AppInfoPtr>* out_app_infos);

 private:
  AppRegistry* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(AppRegistryAsyncWaiter);
};

class  AppRegistryProxy
    : public AppRegistry {
 public:
  explicit AppRegistryProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetApps(GetAppsCallback callback) final;
  void SetAppPreferred(const std::string& app_id, ::apps::mojom::AppPreferred state) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  AppRegistryStubDispatch {
 public:
  static bool Accept(AppRegistry* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      AppRegistry* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<AppRegistry>>
class AppRegistryStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  AppRegistryStub() {}
  ~AppRegistryStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AppRegistryStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AppRegistryStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  AppRegistryRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  AppRegistryResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace apps

namespace mojo {

}  // namespace mojo

#endif  // CHROME_BROWSER_APPS_FOUNDATION_APP_SERVICE_PUBLIC_MOJOM_APP_REGISTRY_MOJOM_H_