// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_BROWSER_APPS_FOUNDATION_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_SHARED_H_
#define CHROME_BROWSER_APPS_FOUNDATION_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "chrome/browser/apps/foundation/app_service/public/mojom/types.mojom-shared-internal.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace apps {
namespace mojom {
class AppInfoDataView;



}  // namespace mojom
}  // namespace apps

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::apps::mojom::AppInfoDataView> {
  using Data = ::apps::mojom::internal::AppInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace apps {
namespace mojom {

enum class AppPreferred : int32_t {
  kUnknown,
  kPreferred,
  kNotPreferred,
  kMinValue = 0,
  kMaxValue = 2,
};

inline std::ostream& operator<<(std::ostream& os, AppPreferred value) {
  switch(value) {
    case AppPreferred::kUnknown:
      return os << "AppPreferred::kUnknown";
    case AppPreferred::kPreferred:
      return os << "AppPreferred::kPreferred";
    case AppPreferred::kNotPreferred:
      return os << "AppPreferred::kNotPreferred";
    default:
      return os << "Unknown AppPreferred value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(AppPreferred value) {
  return internal::AppPreferred_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class AppType : int32_t {
  kUnknown,
  kArc,
  kWeb,
  kCrostini,
  kMinValue = 0,
  kMaxValue = 3,
};

inline std::ostream& operator<<(std::ostream& os, AppType value) {
  switch(value) {
    case AppType::kUnknown:
      return os << "AppType::kUnknown";
    case AppType::kArc:
      return os << "AppType::kArc";
    case AppType::kWeb:
      return os << "AppType::kWeb";
    case AppType::kCrostini:
      return os << "AppType::kCrostini";
    default:
      return os << "Unknown AppType value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(AppType value) {
  return internal::AppType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
class AppInfoDataView {
 public:
  AppInfoDataView() {}

  AppInfoDataView(
      internal::AppInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadId(UserType* output) {
    auto* pointer = data_->id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::apps::mojom::AppType>(
        data_value, output);
  }

  AppType type() const {
    return static_cast<AppType>(data_->type);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPreferred(UserType* output) const {
    auto data_value = data_->preferred;
    return mojo::internal::Deserialize<::apps::mojom::AppPreferred>(
        data_value, output);
  }

  AppPreferred preferred() const {
    return static_cast<AppPreferred>(data_->preferred);
  }
 private:
  internal::AppInfo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace apps

namespace std {

template <>
struct hash<::apps::mojom::AppPreferred>
    : public mojo::internal::EnumHashImpl<::apps::mojom::AppPreferred> {};

template <>
struct hash<::apps::mojom::AppType>
    : public mojo::internal::EnumHashImpl<::apps::mojom::AppType> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::apps::mojom::AppPreferred, ::apps::mojom::AppPreferred> {
  static ::apps::mojom::AppPreferred ToMojom(::apps::mojom::AppPreferred input) { return input; }
  static bool FromMojom(::apps::mojom::AppPreferred input, ::apps::mojom::AppPreferred* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::AppPreferred, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::apps::mojom::AppPreferred, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::apps::mojom::AppPreferred>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::apps::mojom::AppType, ::apps::mojom::AppType> {
  static ::apps::mojom::AppType ToMojom(::apps::mojom::AppType input) { return input; }
  static bool FromMojom(::apps::mojom::AppType input, ::apps::mojom::AppType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::AppType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::apps::mojom::AppType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::apps::mojom::AppType>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::AppInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::apps::mojom::AppInfoDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::apps::mojom::internal::AppInfo_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::id, input, custom_context)) in_id = CallWithContext(Traits::id, input, custom_context);
    typename decltype((*output)->id)::BaseType::BufferWriter
        id_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_id, buffer, &id_writer, context);
    (*output)->id.Set(
        id_writer.is_null() ? nullptr : id_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null id in AppInfo struct");
    mojo::internal::Serialize<::apps::mojom::AppType>(
        CallWithContext(Traits::type, input, custom_context), &(*output)->type);
    mojo::internal::Serialize<::apps::mojom::AppPreferred>(
        CallWithContext(Traits::preferred, input, custom_context), &(*output)->preferred);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::apps::mojom::internal::AppInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::apps::mojom::AppInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace apps {
namespace mojom {

inline void AppInfoDataView::GetIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->id.Get();
  *output = mojo::StringDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace apps

#endif  // CHROME_BROWSER_APPS_FOUNDATION_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_SHARED_H_
