// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_BROWSER_APPS_FOUNDATION_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_H_
#define CHROME_BROWSER_APPS_FOUNDATION_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "chrome/browser/apps/foundation/app_service/public/mojom/types.mojom-shared.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace apps {
namespace mojom {
class AppInfo;
using AppInfoPtr = mojo::InlinedStructPtr<AppInfo>;



class  AppInfo {
 public:
  using DataView = AppInfoDataView;
  using Data_ = internal::AppInfo_Data;

  template <typename... Args>
  static AppInfoPtr New(Args&&... args) {
    return AppInfoPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static AppInfoPtr From(const U& u) {
    return mojo::TypeConverter<AppInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, AppInfo>::Convert(*this);
  }


  AppInfo();

  AppInfo(
      const std::string& id,
      AppType type,
      AppPreferred preferred);

  ~AppInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = AppInfoPtr>
  AppInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, AppInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        AppInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        AppInfo::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::AppInfo_UnserializedMessageContext<
            UserType, AppInfo::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<AppInfo::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return AppInfo::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::AppInfo_UnserializedMessageContext<
            UserType, AppInfo::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<AppInfo::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  std::string id;
  AppType type;
  AppPreferred preferred;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};




template <typename StructPtrType>
AppInfoPtr AppInfo::Clone() const {
  return New(
      mojo::Clone(id),
      mojo::Clone(type),
      mojo::Clone(preferred)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, AppInfo>::value>::type*>
bool AppInfo::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->id, other_struct.id))
    return false;
  if (!mojo::Equals(this->type, other_struct.type))
    return false;
  if (!mojo::Equals(this->preferred, other_struct.preferred))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace apps

namespace mojo {


template <>
struct  StructTraits<::apps::mojom::AppInfo::DataView,
                                         ::apps::mojom::AppInfoPtr> {
  static bool IsNull(const ::apps::mojom::AppInfoPtr& input) { return !input; }
  static void SetToNull(::apps::mojom::AppInfoPtr* output) { output->reset(); }

  static const decltype(::apps::mojom::AppInfo::id)& id(
      const ::apps::mojom::AppInfoPtr& input) {
    return input->id;
  }

  static decltype(::apps::mojom::AppInfo::type) type(
      const ::apps::mojom::AppInfoPtr& input) {
    return input->type;
  }

  static decltype(::apps::mojom::AppInfo::preferred) preferred(
      const ::apps::mojom::AppInfoPtr& input) {
    return input->preferred;
  }

  static bool Read(::apps::mojom::AppInfo::DataView input, ::apps::mojom::AppInfoPtr* output);
};

}  // namespace mojo

#endif  // CHROME_BROWSER_APPS_FOUNDATION_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_H_