// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_COMMON_AVAILABLE_OFFLINE_CONTENT_MOJOM_H_
#define CHROME_COMMON_AVAILABLE_OFFLINE_CONTENT_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "chrome/common/available_offline_content.mojom-shared.h"
#include "url/mojom/url.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace chrome {
namespace mojom {
class AvailableOfflineContentProvider;
using AvailableOfflineContentProviderPtr = mojo::InterfacePtr<AvailableOfflineContentProvider>;
using AvailableOfflineContentProviderPtrInfo = mojo::InterfacePtrInfo<AvailableOfflineContentProvider>;
using ThreadSafeAvailableOfflineContentProviderPtr =
    mojo::ThreadSafeInterfacePtr<AvailableOfflineContentProvider>;
using AvailableOfflineContentProviderRequest = mojo::InterfaceRequest<AvailableOfflineContentProvider>;
using AvailableOfflineContentProviderAssociatedPtr =
    mojo::AssociatedInterfacePtr<AvailableOfflineContentProvider>;
using ThreadSafeAvailableOfflineContentProviderAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<AvailableOfflineContentProvider>;
using AvailableOfflineContentProviderAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<AvailableOfflineContentProvider>;
using AvailableOfflineContentProviderAssociatedRequest =
    mojo::AssociatedInterfaceRequest<AvailableOfflineContentProvider>;

class AvailableOfflineContent;
using AvailableOfflineContentPtr = mojo::StructPtr<AvailableOfflineContent>;

class AvailableOfflineContentSummary;
using AvailableOfflineContentSummaryPtr = mojo::StructPtr<AvailableOfflineContentSummary>;


class AvailableOfflineContentProviderProxy;

template <typename ImplRefTraits>
class AvailableOfflineContentProviderStub;

class AvailableOfflineContentProviderRequestValidator;
class AvailableOfflineContentProviderResponseValidator;

class  AvailableOfflineContentProvider
    : public AvailableOfflineContentProviderInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = AvailableOfflineContentProviderProxy;

  template <typename ImplRefTraits>
  using Stub_ = AvailableOfflineContentProviderStub<ImplRefTraits>;

  using RequestValidator_ = AvailableOfflineContentProviderRequestValidator;
  using ResponseValidator_ = AvailableOfflineContentProviderResponseValidator;
  enum MethodMinVersions : uint32_t {
    kListMinVersion = 0,
    kSummarizeMinVersion = 0,
    kLaunchItemMinVersion = 0,
    kLaunchDownloadsPageMinVersion = 0,
  };
  virtual ~AvailableOfflineContentProvider() {}


  using ListCallback = base::OnceCallback<void(std::vector<AvailableOfflineContentPtr>)>;
  virtual void List(ListCallback callback) = 0;


  using SummarizeCallback = base::OnceCallback<void(AvailableOfflineContentSummaryPtr)>;
  virtual void Summarize(SummarizeCallback callback) = 0;

  virtual void LaunchItem(const std::string& item_id, const std::string& name_space) = 0;

  virtual void LaunchDownloadsPage() = 0;
};
class  AvailableOfflineContentProviderInterceptorForTesting : public AvailableOfflineContentProvider {
  virtual AvailableOfflineContentProvider* GetForwardingInterface() = 0;
  void List(ListCallback callback) override;
  void Summarize(SummarizeCallback callback) override;
  void LaunchItem(const std::string& item_id, const std::string& name_space) override;
  void LaunchDownloadsPage() override;
};
class  AvailableOfflineContentProviderAsyncWaiter {
 public:
  explicit AvailableOfflineContentProviderAsyncWaiter(AvailableOfflineContentProvider* proxy);
  ~AvailableOfflineContentProviderAsyncWaiter();
  void List(
      std::vector<AvailableOfflineContentPtr>* out_out);
  void Summarize(
      AvailableOfflineContentSummaryPtr* out_out);

 private:
  AvailableOfflineContentProvider* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(AvailableOfflineContentProviderAsyncWaiter);
};

class  AvailableOfflineContentProviderProxy
    : public AvailableOfflineContentProvider {
 public:
  explicit AvailableOfflineContentProviderProxy(mojo::MessageReceiverWithResponder* receiver);
  void List(ListCallback callback) final;
  void Summarize(SummarizeCallback callback) final;
  void LaunchItem(const std::string& item_id, const std::string& name_space) final;
  void LaunchDownloadsPage() final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  AvailableOfflineContentProviderStubDispatch {
 public:
  static bool Accept(AvailableOfflineContentProvider* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      AvailableOfflineContentProvider* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<AvailableOfflineContentProvider>>
class AvailableOfflineContentProviderStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  AvailableOfflineContentProviderStub() {}
  ~AvailableOfflineContentProviderStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AvailableOfflineContentProviderStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AvailableOfflineContentProviderStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  AvailableOfflineContentProviderRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  AvailableOfflineContentProviderResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};






class  AvailableOfflineContent {
 public:
  using DataView = AvailableOfflineContentDataView;
  using Data_ = internal::AvailableOfflineContent_Data;

  template <typename... Args>
  static AvailableOfflineContentPtr New(Args&&... args) {
    return AvailableOfflineContentPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static AvailableOfflineContentPtr From(const U& u) {
    return mojo::TypeConverter<AvailableOfflineContentPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, AvailableOfflineContent>::Convert(*this);
  }


  AvailableOfflineContent();

  AvailableOfflineContent(
      const std::string& id,
      const std::string& name_space,
      const std::string& title,
      const std::string& snippet,
      const std::string& date_modified,
      const std::string& attribution,
      const GURL& thumbnail_data_uri,
      AvailableContentType content_type);

  ~AvailableOfflineContent();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = AvailableOfflineContentPtr>
  AvailableOfflineContentPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, AvailableOfflineContent>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        AvailableOfflineContent::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        AvailableOfflineContent::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::AvailableOfflineContent_UnserializedMessageContext<
            UserType, AvailableOfflineContent::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<AvailableOfflineContent::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return AvailableOfflineContent::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::AvailableOfflineContent_UnserializedMessageContext<
            UserType, AvailableOfflineContent::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<AvailableOfflineContent::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  std::string id;
  std::string name_space;
  std::string title;
  std::string snippet;
  std::string date_modified;
  std::string attribution;
  GURL thumbnail_data_uri;
  AvailableContentType content_type;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


class  AvailableOfflineContentSummary {
 public:
  using DataView = AvailableOfflineContentSummaryDataView;
  using Data_ = internal::AvailableOfflineContentSummary_Data;

  template <typename... Args>
  static AvailableOfflineContentSummaryPtr New(Args&&... args) {
    return AvailableOfflineContentSummaryPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static AvailableOfflineContentSummaryPtr From(const U& u) {
    return mojo::TypeConverter<AvailableOfflineContentSummaryPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, AvailableOfflineContentSummary>::Convert(*this);
  }


  AvailableOfflineContentSummary();

  AvailableOfflineContentSummary(
      uint32_t total_items,
      bool has_prefetched_page,
      bool has_offline_page,
      bool has_video,
      bool has_audio);

  ~AvailableOfflineContentSummary();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = AvailableOfflineContentSummaryPtr>
  AvailableOfflineContentSummaryPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, AvailableOfflineContentSummary>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        AvailableOfflineContentSummary::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        AvailableOfflineContentSummary::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::AvailableOfflineContentSummary_UnserializedMessageContext<
            UserType, AvailableOfflineContentSummary::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<AvailableOfflineContentSummary::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return AvailableOfflineContentSummary::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::AvailableOfflineContentSummary_UnserializedMessageContext<
            UserType, AvailableOfflineContentSummary::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<AvailableOfflineContentSummary::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  uint32_t total_items;
  bool has_prefetched_page;
  bool has_offline_page;
  bool has_video;
  bool has_audio;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

template <typename StructPtrType>
AvailableOfflineContentPtr AvailableOfflineContent::Clone() const {
  return New(
      mojo::Clone(id),
      mojo::Clone(name_space),
      mojo::Clone(title),
      mojo::Clone(snippet),
      mojo::Clone(date_modified),
      mojo::Clone(attribution),
      mojo::Clone(thumbnail_data_uri),
      mojo::Clone(content_type)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, AvailableOfflineContent>::value>::type*>
bool AvailableOfflineContent::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->id, other_struct.id))
    return false;
  if (!mojo::Equals(this->name_space, other_struct.name_space))
    return false;
  if (!mojo::Equals(this->title, other_struct.title))
    return false;
  if (!mojo::Equals(this->snippet, other_struct.snippet))
    return false;
  if (!mojo::Equals(this->date_modified, other_struct.date_modified))
    return false;
  if (!mojo::Equals(this->attribution, other_struct.attribution))
    return false;
  if (!mojo::Equals(this->thumbnail_data_uri, other_struct.thumbnail_data_uri))
    return false;
  if (!mojo::Equals(this->content_type, other_struct.content_type))
    return false;
  return true;
}
template <typename StructPtrType>
AvailableOfflineContentSummaryPtr AvailableOfflineContentSummary::Clone() const {
  return New(
      mojo::Clone(total_items),
      mojo::Clone(has_prefetched_page),
      mojo::Clone(has_offline_page),
      mojo::Clone(has_video),
      mojo::Clone(has_audio)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, AvailableOfflineContentSummary>::value>::type*>
bool AvailableOfflineContentSummary::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->total_items, other_struct.total_items))
    return false;
  if (!mojo::Equals(this->has_prefetched_page, other_struct.has_prefetched_page))
    return false;
  if (!mojo::Equals(this->has_offline_page, other_struct.has_offline_page))
    return false;
  if (!mojo::Equals(this->has_video, other_struct.has_video))
    return false;
  if (!mojo::Equals(this->has_audio, other_struct.has_audio))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace chrome

namespace mojo {


template <>
struct  StructTraits<::chrome::mojom::AvailableOfflineContent::DataView,
                                         ::chrome::mojom::AvailableOfflineContentPtr> {
  static bool IsNull(const ::chrome::mojom::AvailableOfflineContentPtr& input) { return !input; }
  static void SetToNull(::chrome::mojom::AvailableOfflineContentPtr* output) { output->reset(); }

  static const decltype(::chrome::mojom::AvailableOfflineContent::id)& id(
      const ::chrome::mojom::AvailableOfflineContentPtr& input) {
    return input->id;
  }

  static const decltype(::chrome::mojom::AvailableOfflineContent::name_space)& name_space(
      const ::chrome::mojom::AvailableOfflineContentPtr& input) {
    return input->name_space;
  }

  static const decltype(::chrome::mojom::AvailableOfflineContent::title)& title(
      const ::chrome::mojom::AvailableOfflineContentPtr& input) {
    return input->title;
  }

  static const decltype(::chrome::mojom::AvailableOfflineContent::snippet)& snippet(
      const ::chrome::mojom::AvailableOfflineContentPtr& input) {
    return input->snippet;
  }

  static const decltype(::chrome::mojom::AvailableOfflineContent::date_modified)& date_modified(
      const ::chrome::mojom::AvailableOfflineContentPtr& input) {
    return input->date_modified;
  }

  static const decltype(::chrome::mojom::AvailableOfflineContent::attribution)& attribution(
      const ::chrome::mojom::AvailableOfflineContentPtr& input) {
    return input->attribution;
  }

  static const decltype(::chrome::mojom::AvailableOfflineContent::thumbnail_data_uri)& thumbnail_data_uri(
      const ::chrome::mojom::AvailableOfflineContentPtr& input) {
    return input->thumbnail_data_uri;
  }

  static decltype(::chrome::mojom::AvailableOfflineContent::content_type) content_type(
      const ::chrome::mojom::AvailableOfflineContentPtr& input) {
    return input->content_type;
  }

  static bool Read(::chrome::mojom::AvailableOfflineContent::DataView input, ::chrome::mojom::AvailableOfflineContentPtr* output);
};


template <>
struct  StructTraits<::chrome::mojom::AvailableOfflineContentSummary::DataView,
                                         ::chrome::mojom::AvailableOfflineContentSummaryPtr> {
  static bool IsNull(const ::chrome::mojom::AvailableOfflineContentSummaryPtr& input) { return !input; }
  static void SetToNull(::chrome::mojom::AvailableOfflineContentSummaryPtr* output) { output->reset(); }

  static decltype(::chrome::mojom::AvailableOfflineContentSummary::total_items) total_items(
      const ::chrome::mojom::AvailableOfflineContentSummaryPtr& input) {
    return input->total_items;
  }

  static decltype(::chrome::mojom::AvailableOfflineContentSummary::has_prefetched_page) has_prefetched_page(
      const ::chrome::mojom::AvailableOfflineContentSummaryPtr& input) {
    return input->has_prefetched_page;
  }

  static decltype(::chrome::mojom::AvailableOfflineContentSummary::has_offline_page) has_offline_page(
      const ::chrome::mojom::AvailableOfflineContentSummaryPtr& input) {
    return input->has_offline_page;
  }

  static decltype(::chrome::mojom::AvailableOfflineContentSummary::has_video) has_video(
      const ::chrome::mojom::AvailableOfflineContentSummaryPtr& input) {
    return input->has_video;
  }

  static decltype(::chrome::mojom::AvailableOfflineContentSummary::has_audio) has_audio(
      const ::chrome::mojom::AvailableOfflineContentSummaryPtr& input) {
    return input->has_audio;
  }

  static bool Read(::chrome::mojom::AvailableOfflineContentSummary::DataView input, ::chrome::mojom::AvailableOfflineContentSummaryPtr* output);
};

}  // namespace mojo

#endif  // CHROME_COMMON_AVAILABLE_OFFLINE_CONTENT_MOJOM_H_