// Copyright 2017 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// vector_icons.cc.template is used to generate vector_icons.cc. Edit the former
// rather than the latter.

#include "components/omnibox/browser/vector_icons.h"

#include "base/logging.h"
#include "ui/gfx/vector_icon_types.h"

#include "components/vector_icons/cc_macros.h"

namespace omnibox {

using namespace gfx;

VECTOR_ICON_REP_TEMPLATE(kCalculatorPath, CANVAS_DIMENSIONS, 32,
MOVE_TO, 8, 12,
LINE_TO, 24, 12,
LINE_TO, 24, 15,
LINE_TO, 8, 15,
LINE_TO, 8, 12,
LINE_TO, 8, 12,
CLOSE,
MOVE_TO, 8, 18,
LINE_TO, 24, 18,
LINE_TO, 24, 21,
LINE_TO, 8, 21,
LINE_TO, 8, 18,
LINE_TO, 8, 18,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kCalculator16Path, CANVAS_DIMENSIONS, 16,
MOVE_TO, 4, 5,
LINE_TO, 12, 5,
LINE_TO, 12, 7,
LINE_TO, 4, 7,
LINE_TO, 4, 5,
LINE_TO, 4, 5,
CLOSE,
MOVE_TO, 4, 9,
LINE_TO, 12, 9,
LINE_TO, 12, 11,
LINE_TO, 4, 11,
LINE_TO, 4, 9,
LINE_TO, 4, 9,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kCalculatorRepList, kCalculatorIcon, {kCalculatorPath, arraysize(kCalculatorPath)}, {kCalculator16Path, arraysize(kCalculator16Path)})
VECTOR_ICON_REP_TEMPLATE(kDriveSheetsPath, CANVAS_DIMENSIONS, 18,
PATH_COLOR_ARGB, 0xFF, 0x0F, 0x9D, 0x58,
MOVE_TO, 16, 0,
H_LINE_TO, 2,
CUBIC_TO, 0.9f, 0, 0.01f, 0.9f, 0.01f, 2,
LINE_TO, 0, 5,
R_V_LINE_TO, 11,
R_CUBIC_TO, 0, 1.1f, 0.9f, 2, 2, 2,
R_H_LINE_TO, 14,
R_CUBIC_TO, 1.1f, 0, 2, -0.9f, 2, -2,
V_LINE_TO, 2,
CUBIC_TO, 18, 0.9f, 17.1f, 0, 16, 0,
CLOSE,
R_MOVE_TO, 0, 8,
H_LINE_TO, 8,
R_V_LINE_TO, 8,
H_LINE_TO, 6,
V_LINE_TO, 8,
H_LINE_TO, 2,
V_LINE_TO, 6,
H_LINE_TO, 6,
V_LINE_TO, 2,
R_H_LINE_TO, 2,
R_V_LINE_TO, 4,
R_H_LINE_TO, 8,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDriveSheetsRepList, kDriveSheetsIcon, {kDriveSheetsPath, arraysize(kDriveSheetsPath)})
VECTOR_ICON_REP_TEMPLATE(kTouchableClearPath, CANVAS_DIMENSIONS, 26,
MOVE_TO, 7.92f, 20.5f,
R_ARC_TO, 126.99f, 126.99f, 0, 0, 1, -5.77f, -5.01f,
R_ARC_TO, 3.27f, 3.27f, 0, 0, 1, -0.3f, -4.7f,
R_CUBIC_TO, 0.06f, -0.07f, 2.08f, -1.82f, 6.06f, -5.24f,
R_CUBIC_TO, 0.71f, -0.61f, 1.52f, -0.54f, 2.54f, -0.54f,
R_CUBIC_TO, 0.01f, -0.01f, 2.04f, -0.01f, 6.1f, 0,
CUBIC_TO, 21.28f, 5, 24.99f, 8.38f, 25, 12.97f,
CUBIC_TO, 25, 17.6f, 21.29f, 21, 16.55f, 21,
R_H_LINE_TO, -6.13f,
R_CUBIC_TO, -0.98f, 0, -2, 0, -2.5f, -0.5f,
CLOSE,
MOVE_TO, 21, 9.97f,
LINE_TO, 20.03f, 9,
LINE_TO, 17, 12.03f,
LINE_TO, 13.97f, 9,
R_LINE_TO, -0.97f, 0.97f,
LINE_TO, 16.03f, 13,
LINE_TO, 13, 16.03f,
R_LINE_TO, 0.97f, 0.97f,
LINE_TO, 17, 13.97f,
LINE_TO, 20.03f, 17,
R_LINE_TO, 0.97f, -0.97f,
LINE_TO, 17.97f, 13,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTouchableClearRepList, kTouchableClearIcon, {kTouchableClearPath, arraysize(kTouchableClearPath)})
VECTOR_ICON_REP_TEMPLATE(kTabPath, CANVAS_DIMENSIONS, 32,
MOVE_TO, 3, 26,
LINE_TO, 7, 12,
LINE_TO, 24, 12,
LINE_TO, 28, 26,
LINE_TO, 24, 26,
LINE_TO, 22, 16,
LINE_TO, 9, 16,
LINE_TO, 7, 26,
LINE_TO, 3, 26,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTabRepList, kTabIcon, {kTabPath, arraysize(kTabPath)})
VECTOR_ICON_REP_TEMPLATE(kBlankPath, CANVAS_DIMENSIONS, 32)
VECTOR_ICON_REP_TEMPLATE(kBlank16Path, CANVAS_DIMENSIONS, 16)
VECTOR_ICON_TEMPLATE_CC(kBlankRepList, kBlankIcon, {kBlankPath, arraysize(kBlankPath)}, {kBlank16Path, arraysize(kBlank16Path)})
VECTOR_ICON_REP_TEMPLATE(kMdPagePath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 12, 2,
R_CUBIC_TO, 5.52f, 0, 10, 4.48f, 10, 10,
R_CUBIC_TO, 0, 5.52f, -4.48f, 10, -10, 10,
CUBIC_TO_SHORTHAND, 2, 17.52f, 2, 12,
CUBIC_TO_SHORTHAND, 6.48f, 2, 12, 2,
CLOSE,
MOVE_TO, 4, 12,
R_H_LINE_TO, 4.4f,
R_CUBIC_TO, 3.41f, 0.02f, 4.92f, 1.73f, 4.54f, 5.13f,
H_LINE_TO, 9.49f,
R_V_LINE_TO, 2.47f,
R_ARC_TO, 8, 8, 0, 0, 0, 10.5f, -8.08f,
CUBIC_TO, 19.33f, 12.5f, 18.33f, 13, 17, 13,
R_CUBIC_TO, -2.14f, 0, -3.21f, -0.92f, -3.21f, -2.75f,
R_H_LINE_TO, -3.75f,
R_CUBIC_TO, -0.27f, -2.73f, 0.68f, -4.09f, 2.87f, -4.09f,
R_CUBIC_TO, 0, -0.97f, 0.33f, -1.6f, 0.81f, -1.97f,
ARC_TO, 8, 8, 0, 0, 0, 4, 12,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kMdPageRepList, kMdPageIcon, {kMdPagePath, arraysize(kMdPagePath)})
VECTOR_ICON_REP_TEMPLATE(kDriveLogoPath, CANVAS_DIMENSIONS, 192,
PATH_COLOR_ARGB, 0xFF, 0x42, 0x85, 0xF4,
MOVE_TO, 33, 124,
R_V_LINE_TO, 52,
R_H_LINE_TO, 126,
R_LINE_TO, 29, -52,
H_LINE_TO, 33,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x0F, 0x9D, 0x58,
MOVE_TO, 66, 16,
LINE_TO, 4, 124,
R_LINE_TO, 29, 52,
R_LINE_TO, 71, -122,
R_LINE_TO, -38, -38,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFC, 0xB7, 0x00,
MOVE_TO, 126, 16,
H_LINE_TO, 66,
R_LINE_TO, 63, 108,
R_H_LINE_TO, 59,
LINE_TO, 126, 16,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDriveLogoRepList, kDriveLogoIcon, {kDriveLogoPath, arraysize(kDriveLogoPath)})
VECTOR_ICON_REP_TEMPLATE(kDriveDocsPath, CANVAS_DIMENSIONS, 18,
PATH_COLOR_ARGB, 0xFF, 0x42, 0x85, 0xF4,
MOVE_TO, 16, 0,
H_LINE_TO, 2,
CUBIC_TO, 0.9f, 0, 0, 0.9f, 0, 2,
R_V_LINE_TO, 14,
R_CUBIC_TO, 0, 1.1f, 0.9f, 2, 2, 2,
R_H_LINE_TO, 14,
R_CUBIC_TO, 1.1f, 0, 2, -0.9f, 2, -2,
V_LINE_TO, 2,
R_CUBIC_TO, 0, -1.1f, -0.9f, -2, -2, -2,
CLOSE,
R_MOVE_TO, -1.99f, 6,
H_LINE_TO, 4,
V_LINE_TO, 4,
R_H_LINE_TO, 10.01f,
CLOSE,
R_MOVE_TO, 0, 4,
H_LINE_TO, 4,
V_LINE_TO, 8,
R_H_LINE_TO, 10.01f,
CLOSE,
R_MOVE_TO, -3, 4,
H_LINE_TO, 4,
R_V_LINE_TO, -2,
R_H_LINE_TO, 7.01f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDriveDocsRepList, kDriveDocsIcon, {kDriveDocsPath, arraysize(kDriveDocsPath)})
VECTOR_ICON_REP_TEMPLATE(kKeywordSearchPath, CANVAS_DIMENSIONS, 32,
MOVE_TO, 21.5f, 14.94f,
R_CUBIC_TO, 0.31f, 0.27f, 0.5f, 0.65f, 0.5f, 1.06f,
R_CUBIC_TO, 0, 0.41f, -0.19f, 0.79f, -0.5f, 1.06f,
R_LINE_TO, -8.57f, 7.5f,
R_CUBIC_TO, -0.31f, 0.27f, -0.74f, 0.44f, -1.21f, 0.44f,
R_CUBIC_TO, -0.95f, 0, -1.71f, -0.67f, -1.71f, -1.5f,
R_CUBIC_TO, 0, -0.41f, 0.19f, -0.79f, 0.5f, -1.06f,
R_LINE_TO, 7.36f, -6.44f,
R_LINE_TO, -7.36f, -6.44f,
CUBIC_TO, 10.19f, 9.29f, 10, 8.91f, 10, 8.5f,
R_CUBIC_TO, 0, -0.83f, 0.77f, -1.5f, 1.71f, -1.5f,
R_CUBIC_TO, 0.47f, 0, 0.9f, 0.17f, 1.21f, 0.44f,
R_LINE_TO, 8.57f, 7.5f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kKeywordSearchRepList, kKeywordSearchIcon, {kKeywordSearchPath, arraysize(kKeywordSearchPath)})
VECTOR_ICON_REP_TEMPLATE(kDriveSlidesPath, CANVAS_DIMENSIONS, 18,
PATH_COLOR_ARGB, 0xFF, 0xFC, 0xB7, 0x00,
MOVE_TO, 16, 0,
R_H_LINE_TO, -14,
CUBIC_TO, 0.9f, 0, 0, 0.9f, 0, 2,
R_V_LINE_TO, 14,
R_CUBIC_TO, 0, 1.1f, 0.9f, 2, 2, 2,
R_H_LINE_TO, 14,
R_CUBIC_TO, 1.1f, 0, 2, -0.9f, 2, -2,
V_LINE_TO, 2,
R_CUBIC_TO, 0, -1.1f, -0.9f, -2, -2, -2,
CLOSE,
R_MOVE_TO, 0, 13,
R_H_LINE_TO, -14,
V_LINE_TO, 5,
R_H_LINE_TO, 14,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDriveSlidesRepList, kDriveSlidesIcon, {kDriveSlidesPath, arraysize(kDriveSlidesPath)})
VECTOR_ICON_REP_TEMPLATE(kAnswerDictionaryPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 1, 13,
R_H_LINE_TO, 14,
R_V_LINE_TO, 2,
H_LINE_TO, 1,
CLOSE,
MOVE_TO, 13, 1,
R_H_LINE_TO, 1,
R_V_LINE_TO, 10,
R_H_LINE_TO, -1,
CLOSE,
MOVE_TO, 5.81f, 1,
LINE_TO, 2, 11,
R_H_LINE_TO, 1.56f,
R_LINE_TO, 0.78f, -2.14f,
R_H_LINE_TO, 4.33f,
LINE_TO, 9.44f, 11,
H_LINE_TO, 11,
LINE_TO, 7.2f, 1,
H_LINE_TO, 5.81f,
CLOSE,
R_MOVE_TO, -0.95f, 6.43f,
R_LINE_TO, 1.65f, -4.52f,
R_LINE_TO, 1.65f, 4.52f,
H_LINE_TO, 4.85f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kAnswerDictionaryRepList, kAnswerDictionaryIcon, {kAnswerDictionaryPath, arraysize(kAnswerDictionaryPath)})
VECTOR_ICON_REP_TEMPLATE(kStarPath, CANVAS_DIMENSIONS, 32,
MOVE_TO, 13.27f, 12.67f,
LINE_TO, 5, 13.39f,
LINE_TO, 11.27f, 18.86f,
LINE_TO, 9.39f, 27,
LINE_TO, 16.5f, 22.68f,
LINE_TO, 23.61f, 27,
LINE_TO, 21.73f, 18.86f,
LINE_TO, 28, 13.39f,
LINE_TO, 19.73f, 12.67f,
LINE_TO, 16.5f, 5,
LINE_TO, 13.27f, 12.67f,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kStar16Path, CANVAS_DIMENSIONS, 16,
MOVE_TO, 6.31f, 6.19f,
LINE_TO, 2, 6.58f,
LINE_TO, 5.27f, 9.56f,
LINE_TO, 4.29f, 14,
LINE_TO, 8, 11.65f,
LINE_TO, 11.71f, 14,
LINE_TO, 10.73f, 9.56f,
LINE_TO, 14, 6.58f,
LINE_TO, 9.69f, 6.19f,
LINE_TO, 8, 2,
LINE_TO, 6.31f, 6.19f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kStarRepList, kStarIcon, {kStarPath, arraysize(kStarPath)}, {kStar16Path, arraysize(kStar16Path)})
VECTOR_ICON_REP_TEMPLATE(kAnswerDefaultPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 8.14f, 9.33f,
V_LINE_TO, 6.73f,
R_H_LINE_TO, 6.68f,
R_CUBIC_TO, 0.1f, 0.44f, 0.18f, 0.85f, 0.18f, 1.44f,
CUBIC_TO, 15, 12.16f, 12.27f, 15, 8.15f, 15,
CUBIC_TO, 4.2f, 15, 1, 11.86f, 1, 8,
R_CUBIC_TO, 0, -3.86f, 3.2f, -7, 7.14f, -7,
R_CUBIC_TO, 1.93f, 0, 3.54f, 0.69f, 4.78f, 1.83f,
R_LINE_TO, -2.03f, 1.93f,
R_CUBIC_TO, -0.51f, -0.48f, -1.41f, -1.04f, -2.75f, -1.04f,
ARC_TO, 4.3f, 4.3f, 0, 0, 0, 3.85f, 8,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, 4.29f, 4.28f,
R_CUBIC_TO, 2.74f, 0, 3.74f, -1.85f, 3.93f, -2.95f,
H_LINE_TO, 8.14f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kAnswerDefaultRepList, kAnswerDefaultIcon, {kAnswerDefaultPath, arraysize(kAnswerDefaultPath)})
VECTOR_ICON_REP_TEMPLATE(kTouchablePagePath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 6, 16.93f,
CUBIC_TO, 3.61f, 15.55f, 2, 12.96f, 2, 10,
CUBIC_TO, 2, 5.58f, 5.58f, 2, 10, 2,
CUBIC_TO, 14.42f, 2, 18, 5.58f, 18, 10,
CUBIC_TO, 18, 12.96f, 16.39f, 15.55f, 14, 16.93f,
LINE_TO, 14, 9.5f,
LINE_TO, 10.5f, 6,
LINE_TO, 6, 6,
LINE_TO, 6, 16.93f,
CLOSE,
MOVE_TO, 13, 11,
LINE_TO, 9, 11,
LINE_TO, 9, 7,
LINE_TO, 13, 11,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTouchablePageRepList, kTouchablePageIcon, {kTouchablePagePath, arraysize(kTouchablePagePath)})
VECTOR_ICON_REP_TEMPLATE(kTouchableSearchPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 16.02f, 15.27f,
R_LINE_TO, -2.59f, -2.6f,
R_H_LINE_TO, -0.47f,
R_LINE_TO, -0.17f, -0.16f,
R_ARC_TO, 3.84f, 3.84f, 0, 0, 0, 0.93f, -2.51f,
R_ARC_TO, 3.86f, 3.86f, 0, 0, 0, -3.86f, -3.86f,
ARC_TO, 3.86f, 3.86f, 0, 0, 0, 6, 10,
R_ARC_TO, 3.86f, 3.86f, 0, 0, 0, 6.37f, 2.93f,
R_LINE_TO, 0.16f, 0.17f,
R_V_LINE_TO, 0.47f,
R_LINE_TO, 2.59f, 2.59f,
R_ARC_TO, 8, 8, 0, 1, 1, 0.9f, -0.87f,
CLOSE,
MOVE_TO, 9.86f, 12.67f,
ARC_TO, 2.67f, 2.67f, 0, 0, 1, 7.19f, 10,
ARC_TO, 2.67f, 2.67f, 0, 0, 1, 9.86f, 7.33f,
ARC_TO, 2.67f, 2.67f, 0, 0, 1, 12.53f, 10,
R_ARC_TO, 2.67f, 2.67f, 0, 0, 1, -2.67f, 2.67f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTouchableSearchRepList, kTouchableSearchIcon, {kTouchableSearchPath, arraysize(kTouchableSearchPath)})
VECTOR_ICON_REP_TEMPLATE(kTouchableBookmarkPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 10, 18,
R_ARC_TO, 8, 8, 0, 1, 1, 0, -16,
R_ARC_TO, 8, 8, 0, 0, 1, 0, 16,
CLOSE,
R_MOVE_TO, 0, -4.96f,
LINE_TO, 13.09f, 15,
R_LINE_TO, -0.82f, -3.7f,
LINE_TO, 15, 8.81f,
R_LINE_TO, -3.59f, -0.32f,
LINE_TO, 10, 5,
LINE_TO, 8.6f, 8.49f,
LINE_TO, 5, 8.81f,
R_LINE_TO, 2.73f, 2.49f,
R_LINE_TO, -0.82f, 3.7f,
LINE_TO, 10, 13.04f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTouchableBookmarkRepList, kTouchableBookmarkIcon, {kTouchableBookmarkPath, arraysize(kTouchableBookmarkPath)})
VECTOR_ICON_REP_TEMPLATE(kAnswerFinancePath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 9, 12,
V_LINE_TO, 7,
R_H_LINE_TO, 2,
R_V_LINE_TO, 5,
R_H_LINE_TO, 2,
R_LINE_TO, -3, 3,
R_LINE_TO, -3, -3,
R_H_LINE_TO, 2,
CLOSE,
MOVE_TO, 7, 4,
R_V_LINE_TO, 5,
H_LINE_TO, 5,
V_LINE_TO, 4,
H_LINE_TO, 3,
R_LINE_TO, 3, -3,
R_LINE_TO, 3, 3,
H_LINE_TO, 7,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kAnswerFinanceRepList, kAnswerFinanceIcon, {kAnswerFinancePath, arraysize(kAnswerFinancePath)})
VECTOR_ICON_REP_TEMPLATE(kExtensionAppPath, CANVAS_DIMENSIONS, 32,
MOVE_TO, 24, 14,
LINE_TO, 24, 10,
CUBIC_TO, 24, 9, 23, 8, 22, 8,
LINE_TO, 18, 8,
LINE_TO, 18, 6,
CUBIC_TO, 17.57f, 4.28f, 16.5f, 3, 15, 3,
CUBIC_TO, 13.5f, 3, 11.86f, 4.28f, 12, 6,
LINE_TO, 12, 8,
LINE_TO, 7, 8,
CUBIC_TO, 6, 8, 5, 9, 5, 10,
LINE_TO, 5, 14,
LINE_TO, 7, 14,
CUBIC_TO, 8.42f, 14.2f, 9.8f, 15.58f, 10, 17,
CUBIC_TO, 9.8f, 18.99f, 8.42f, 20.37f, 7, 20,
LINE_TO, 5, 20,
LINE_TO, 5, 25,
CUBIC_TO, 5, 25.97f, 6.03f, 27, 7, 27,
LINE_TO, 12, 27,
LINE_TO, 12, 25,
CUBIC_TO, 11.63f, 23.58f, 13.01f, 22.2f, 15, 22,
CUBIC_TO, 16.42f, 22.2f, 17.8f, 23.58f, 18, 25,
LINE_TO, 18, 27,
LINE_TO, 22, 27,
CUBIC_TO, 23, 27, 24, 26, 24, 25,
LINE_TO, 24, 20,
LINE_TO, 26, 20,
CUBIC_TO, 27.72f, 20.14f, 29, 18.5f, 29, 17,
CUBIC_TO, 29, 15.5f, 27.72f, 14.43f, 26, 14,
LINE_TO, 24, 14,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kExtensionApp16Path, CANVAS_DIMENSIONS, 16,
MOVE_TO, 12.71f, 6.71f,
LINE_TO, 12.71f, 4.43f,
CUBIC_TO, 12.71f, 3.8f, 12.2f, 3.29f, 11.57f, 3.29f,
LINE_TO, 9.29f, 3.29f,
LINE_TO, 9.29f, 2.43f,
CUBIC_TO, 9.29f, 1.64f, 8.65f, 1, 7.86f, 1,
CUBIC_TO, 7.07f, 1, 6.43f, 1.64f, 6.43f, 2.43f,
LINE_TO, 6.43f, 3.29f,
LINE_TO, 4.14f, 3.29f,
CUBIC_TO, 3.51f, 3.29f, 3.01f, 3.8f, 3.01f, 4.43f,
LINE_TO, 3.01f, 6.6f,
LINE_TO, 3.86f, 6.6f,
CUBIC_TO, 4.71f, 6.6f, 5.4f, 7.29f, 5.4f, 8.14f,
CUBIC_TO, 5.4f, 8.99f, 4.71f, 9.69f, 3.86f, 9.69f,
LINE_TO, 3, 9.69f,
LINE_TO, 3, 11.86f,
CUBIC_TO, 3, 12.49f, 3.51f, 13, 4.14f, 13,
LINE_TO, 6.31f, 13,
LINE_TO, 6.31f, 12.14f,
CUBIC_TO, 6.31f, 11.29f, 7.01f, 10.6f, 7.86f, 10.6f,
CUBIC_TO, 8.71f, 10.6f, 9.4f, 11.29f, 9.4f, 12.14f,
LINE_TO, 9.4f, 13,
LINE_TO, 11.57f, 13,
CUBIC_TO, 12.2f, 13, 12.71f, 12.49f, 12.71f, 11.86f,
LINE_TO, 12.71f, 9.57f,
LINE_TO, 13.57f, 9.57f,
CUBIC_TO, 14.36f, 9.57f, 15, 8.93f, 15, 8.14f,
CUBIC_TO, 15, 7.35f, 14.36f, 6.71f, 13.57f, 6.71f,
LINE_TO, 12.71f, 6.71f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kExtensionAppRepList, kExtensionAppIcon, {kExtensionAppPath, arraysize(kExtensionAppPath)}, {kExtensionApp16Path, arraysize(kExtensionApp16Path)})
VECTOR_ICON_REP_TEMPLATE(kAnswerCurrencyPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 8, 3.64f,
R_V_LINE_TO, 2.18f,
R_LINE_TO, 3, -2.91f,
LINE_TO, 8, 0,
R_V_LINE_TO, 2.18f,
CUBIC_TO, 4.68f, 2.18f, 2, 4.79f, 2, 8,
R_CUBIC_TO, 0, 1.14f, 0.35f, 2.2f, 0.93f, 3.1f,
R_LINE_TO, 1.1f, -1.06f,
ARC_TO, 4.17f, 4.17f, 0, 0, 1, 3.5f, 8,
R_CUBIC_TO, 0, -2.41f, 2.02f, -4.36f, 4.5f, -4.36f,
CLOSE,
R_MOVE_TO, 5.07f, 1.27f,
R_LINE_TO, -1.09f, 1.06f,
R_CUBIC_TO, 0.33f, 0.61f, 0.53f, 1.3f, 0.53f, 2.04f,
R_CUBIC_TO, 0, 2.41f, -2.02f, 4.36f, -4.5f, 4.36f,
R_V_LINE_TO, -2.18f,
LINE_TO, 5, 13.09f,
LINE_TO, 8, 16,
R_V_LINE_TO, -2.18f,
R_CUBIC_TO, 3.32f, 0, 6, -2.6f, 6, -5.82f,
R_ARC_TO, 5.64f, 5.64f, 0, 0, 0, -0.93f, -3.1f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kAnswerCurrencyRepList, kAnswerCurrencyIcon, {kAnswerCurrencyPath, arraysize(kAnswerCurrencyPath)})
VECTOR_ICON_REP_TEMPLATE(kAnswerWhenIsPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 8, 0,
ARC_TO, 7.99f, 7.99f, 0, 0, 0, 0, 8,
R_CUBIC_TO, 0, 4.42f, 3.58f, 8, 8, 8,
R_ARC_TO, 8, 8, 0, 1, 0, 0, -16,
CLOSE,
MOVE_TO, 8, 14,
R_CUBIC_TO, -3.31f, 0, -6, -2.68f, -6, -6,
R_CUBIC_TO, 0, -3.32f, 2.69f, -6, 6, -6,
R_CUBIC_TO, 3.31f, 0, 6, 2.69f, 6, 6,
R_CUBIC_TO, 0, 3.31f, -2.68f, 6, -6, 6,
CLOSE,
MOVE_TO, 8, 5,
H_LINE_TO, 7,
R_V_LINE_TO, 3.93f,
LINE_TO, 10.5f, 11,
R_LINE_TO, 0.5f, -0.81f,
R_LINE_TO, -3, -1.75f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kAnswerWhenIsRepList, kAnswerWhenIsIcon, {kAnswerWhenIsPath, arraysize(kAnswerWhenIsPath)})
VECTOR_ICON_REP_TEMPLATE(kHttpPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 4, 2,
LINE_TO, 4, 14,
LINE_TO, 12, 14,
LINE_TO, 12, 5,
LINE_TO, 9, 2,
LINE_TO, 4, 2,
CLOSE,
MOVE_TO, 9, 5,
LINE_TO, 9, 3.25f,
LINE_TO, 10.75f, 5,
LINE_TO, 9, 5,
CLOSE,
MOVE_TO, 5, 13,
LINE_TO, 5, 3,
LINE_TO, 8, 3,
LINE_TO, 8, 6,
LINE_TO, 11, 6,
LINE_TO, 11, 13,
LINE_TO, 5, 13,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kHttpRepList, kHttpIcon, {kHttpPath, arraysize(kHttpPath)})
VECTOR_ICON_REP_TEMPLATE(kSwitchPath, CANVAS_DIMENSIONS, 32,
MOVE_TO, 25.33f, 5.33f,
CUBIC_TO, 26.8f, 5.33f, 28, 6.53f, 28, 8,
R_V_LINE_TO, 16,
R_CUBIC_TO, 0, 1.47f, -1.2f, 2.67f, -2.67f, 2.67f,
H_LINE_TO, 6.67f,
ARC_TO, 2.68f, 2.68f, 0, 0, 1, 4, 24,
V_LINE_TO, 8,
R_CUBIC_TO, 0, -1.47f, 1.2f, -2.67f, 2.67f, -2.67f,
R_H_LINE_TO, 18.67f,
CLOSE,
R_MOVE_TO, 0, 18.67f,
V_LINE_TO, 13.33f,
R_H_LINE_TO, -8,
V_LINE_TO, 8,
H_LINE_TO, 6.67f,
R_V_LINE_TO, 16,
R_H_LINE_TO, 18.67f,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kSwitch16Path, CANVAS_DIMENSIONS, 16,
R_MOVE_TO, 13.44f, 2,
R_CUBIC_TO, 0.86f, 0, 1.56f, 0.68f, 1.56f, 1.5f,
R_V_LINE_TO, 9,
R_CUBIC_TO, 0, 0.83f, -0.7f, 1.5f, -1.56f, 1.5f,
R_H_LINE_TO, -10.89f,
R_CUBIC_TO, -0.86f, 0, -1.56f, -0.67f, -1.56f, -1.5f,
R_V_LINE_TO, -9,
R_CUBIC_TO, 0, -0.82f, 0.7f, -1.5f, 1.56f, -1.5f,
CLOSE,
R_MOVE_TO, -0.44f, 10,
R_V_LINE_TO, -5,
R_H_LINE_TO, -4,
R_V_LINE_TO, -3,
R_H_LINE_TO, -6,
R_V_LINE_TO, 8,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSwitchRepList, kSwitchIcon, {kSwitchPath, arraysize(kSwitchPath)}, {kSwitch16Path, arraysize(kSwitch16Path)})
VECTOR_ICON_REP_TEMPLATE(kAnswerSunrisePath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 8, 4,
CUBIC_TO, 5.79f, 4, 4, 5.79f, 4, 8,
R_CUBIC_TO, 0, 2.21f, 1.79f, 4, 4, 4,
R_CUBIC_TO, 2.21f, 0, 4, -1.79f, 4, -4,
R_CUBIC_TO, 0, -2.21f, -1.79f, -4, -4, -4,
CLOSE,
R_MOVE_TO, 1, -4,
R_V_LINE_TO, 2,
H_LINE_TO, 7,
V_LINE_TO, 0,
R_H_LINE_TO, 2,
CLOSE,
R_MOVE_TO, 0, 14,
R_V_LINE_TO, 2,
H_LINE_TO, 7,
R_V_LINE_TO, -2,
R_H_LINE_TO, 2,
CLOSE,
MOVE_TO, 0, 7,
R_H_LINE_TO, 2,
R_V_LINE_TO, 2,
H_LINE_TO, 0,
V_LINE_TO, 7,
CLOSE,
R_MOVE_TO, 14, 0,
R_H_LINE_TO, 2,
R_V_LINE_TO, 2,
R_H_LINE_TO, -2,
V_LINE_TO, 7,
CLOSE,
MOVE_TO, 3.05f, 1.64f,
LINE_TO, 4.46f, 3.05f,
LINE_TO, 3.05f, 4.46f,
LINE_TO, 1.64f, 3.05f,
LINE_TO, 3.05f, 1.64f,
CLOSE,
R_MOVE_TO, 9.9f, 9.9f,
R_LINE_TO, 1.41f, 1.41f,
R_LINE_TO, -1.41f, 1.41f,
R_LINE_TO, -1.41f, -1.41f,
R_LINE_TO, 1.41f, -1.41f,
CLOSE,
MOVE_TO, 1.64f, 12.95f,
R_LINE_TO, 1.41f, -1.41f,
R_LINE_TO, 1.41f, 1.41f,
R_LINE_TO, -1.41f, 1.41f,
R_LINE_TO, -1.41f, -1.41f,
CLOSE,
R_MOVE_TO, 9.9f, -9.9f,
R_LINE_TO, 1.41f, -1.41f,
R_LINE_TO, 1.41f, 1.41f,
R_LINE_TO, -1.41f, 1.41f,
R_LINE_TO, -1.41f, -1.41f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kAnswerSunriseRepList, kAnswerSunriseIcon, {kAnswerSunrisePath, arraysize(kAnswerSunrisePath)})

}
