// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_AUDIO_FOCUS_MOJOM_SHARED_H_
#define SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_AUDIO_FOCUS_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/media_session/public/mojom/audio_focus.mojom-shared-internal.h"
#include "services/media_session/public/mojom/media_session.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace media_session {
namespace mojom {


}  // namespace mojom
}  // namespace media_session

namespace mojo {
namespace internal {

}  // namespace internal
}  // namespace mojo


namespace media_session {
namespace mojom {

enum class AudioFocusType : int32_t {
  kGain,
  kGainTransientMayDuck,
  kMinValue = 0,
  kMaxValue = 1,
};

inline std::ostream& operator<<(std::ostream& os, AudioFocusType value) {
  switch(value) {
    case AudioFocusType::kGain:
      return os << "AudioFocusType::kGain";
    case AudioFocusType::kGainTransientMayDuck:
      return os << "AudioFocusType::kGainTransientMayDuck";
    default:
      return os << "Unknown AudioFocusType value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(AudioFocusType value) {
  return internal::AudioFocusType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class AudioFocusObserverInterfaceBase {};

using AudioFocusObserverPtrDataView =
    mojo::InterfacePtrDataView<AudioFocusObserverInterfaceBase>;
using AudioFocusObserverRequestDataView =
    mojo::InterfaceRequestDataView<AudioFocusObserverInterfaceBase>;
using AudioFocusObserverAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<AudioFocusObserverInterfaceBase>;
using AudioFocusObserverAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<AudioFocusObserverInterfaceBase>;
class AudioFocusObserver_OnFocusGained_ParamsDataView {
 public:
  AudioFocusObserver_OnFocusGained_ParamsDataView() {}

  AudioFocusObserver_OnFocusGained_ParamsDataView(
      internal::AudioFocusObserver_OnFocusGained_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeSession() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::media_session::mojom::MediaSessionPtrDataView>(
            &data_->session, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::media_session::mojom::AudioFocusType>(
        data_value, output);
  }

  AudioFocusType type() const {
    return static_cast<AudioFocusType>(data_->type);
  }
 private:
  internal::AudioFocusObserver_OnFocusGained_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AudioFocusObserver_OnFocusLost_ParamsDataView {
 public:
  AudioFocusObserver_OnFocusLost_ParamsDataView() {}

  AudioFocusObserver_OnFocusLost_ParamsDataView(
      internal::AudioFocusObserver_OnFocusLost_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeSession() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::media_session::mojom::MediaSessionPtrDataView>(
            &data_->session, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::AudioFocusObserver_OnFocusLost_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace media_session

namespace std {

template <>
struct hash<::media_session::mojom::AudioFocusType>
    : public mojo::internal::EnumHashImpl<::media_session::mojom::AudioFocusType> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::media_session::mojom::AudioFocusType, ::media_session::mojom::AudioFocusType> {
  static ::media_session::mojom::AudioFocusType ToMojom(::media_session::mojom::AudioFocusType input) { return input; }
  static bool FromMojom(::media_session::mojom::AudioFocusType input, ::media_session::mojom::AudioFocusType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::media_session::mojom::AudioFocusType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::media_session::mojom::AudioFocusType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::media_session::mojom::AudioFocusType>(input), output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace media_session {
namespace mojom {






}  // namespace mojom
}  // namespace media_session

#endif  // SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_AUDIO_FOCUS_MOJOM_SHARED_H_
