// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_AUDIO_FOCUS_MOJOM_H_
#define SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_AUDIO_FOCUS_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/media_session/public/mojom/audio_focus.mojom-shared.h"
#include "services/media_session/public/mojom/media_session.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace media_session {
namespace mojom {
class AudioFocusObserver;
using AudioFocusObserverPtr = mojo::InterfacePtr<AudioFocusObserver>;
using AudioFocusObserverPtrInfo = mojo::InterfacePtrInfo<AudioFocusObserver>;
using ThreadSafeAudioFocusObserverPtr =
    mojo::ThreadSafeInterfacePtr<AudioFocusObserver>;
using AudioFocusObserverRequest = mojo::InterfaceRequest<AudioFocusObserver>;
using AudioFocusObserverAssociatedPtr =
    mojo::AssociatedInterfacePtr<AudioFocusObserver>;
using ThreadSafeAudioFocusObserverAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<AudioFocusObserver>;
using AudioFocusObserverAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<AudioFocusObserver>;
using AudioFocusObserverAssociatedRequest =
    mojo::AssociatedInterfaceRequest<AudioFocusObserver>;


class AudioFocusObserverProxy;

template <typename ImplRefTraits>
class AudioFocusObserverStub;

class AudioFocusObserverRequestValidator;

class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM) AudioFocusObserver
    : public AudioFocusObserverInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = AudioFocusObserverProxy;

  template <typename ImplRefTraits>
  using Stub_ = AudioFocusObserverStub<ImplRefTraits>;

  using RequestValidator_ = AudioFocusObserverRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnFocusGainedMinVersion = 0,
    kOnFocusLostMinVersion = 0,
  };
  virtual ~AudioFocusObserver() {}

  virtual void OnFocusGained(::media_session::mojom::MediaSessionPtr session, AudioFocusType type) = 0;

  virtual void OnFocusLost(::media_session::mojom::MediaSessionPtr session) = 0;
};
class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM) AudioFocusObserverInterceptorForTesting : public AudioFocusObserver {
  virtual AudioFocusObserver* GetForwardingInterface() = 0;
  void OnFocusGained(::media_session::mojom::MediaSessionPtr session, AudioFocusType type) override;
  void OnFocusLost(::media_session::mojom::MediaSessionPtr session) override;
};
class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM) AudioFocusObserverAsyncWaiter {
 public:
  explicit AudioFocusObserverAsyncWaiter(AudioFocusObserver* proxy);
  ~AudioFocusObserverAsyncWaiter();

 private:
  AudioFocusObserver* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(AudioFocusObserverAsyncWaiter);
};

class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM) AudioFocusObserverProxy
    : public AudioFocusObserver {
 public:
  explicit AudioFocusObserverProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnFocusGained(::media_session::mojom::MediaSessionPtr session, AudioFocusType type) final;
  void OnFocusLost(::media_session::mojom::MediaSessionPtr session) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM) AudioFocusObserverStubDispatch {
 public:
  static bool Accept(AudioFocusObserver* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      AudioFocusObserver* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<AudioFocusObserver>>
class AudioFocusObserverStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  AudioFocusObserverStub() {}
  ~AudioFocusObserverStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AudioFocusObserverStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AudioFocusObserverStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM) AudioFocusObserverRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace media_session

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_AUDIO_FOCUS_MOJOM_H_