// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_NETWORK_PUBLIC_MOJOM_HOST_RESOLVER_MOJOM_SHARED_H_
#define SERVICES_NETWORK_PUBLIC_MOJOM_HOST_RESOLVER_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/network/public/mojom/host_resolver.mojom-shared-internal.h"
#include "net/interfaces/address_list.mojom-shared.h"
#include "services/network/public/mojom/network_param.mojom-shared.h"
#include "services/network/public/mojom/url_loader.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace network {
namespace mojom {
class ResolveHostParametersDataView;



}  // namespace mojom
}  // namespace network

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::network::mojom::ResolveHostParametersDataView> {
  using Data = ::network::mojom::internal::ResolveHostParameters_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace network {
namespace mojom {

enum class ResolveHostParameters_DnsQueryType : int32_t {
  UNSPECIFIED,
  A,
  AAAA,
  kMinValue = 0,
  kMaxValue = 2,
};

inline std::ostream& operator<<(std::ostream& os, ResolveHostParameters_DnsQueryType value) {
  switch(value) {
    case ResolveHostParameters_DnsQueryType::UNSPECIFIED:
      return os << "ResolveHostParameters_DnsQueryType::UNSPECIFIED";
    case ResolveHostParameters_DnsQueryType::A:
      return os << "ResolveHostParameters_DnsQueryType::A";
    case ResolveHostParameters_DnsQueryType::AAAA:
      return os << "ResolveHostParameters_DnsQueryType::AAAA";
    default:
      return os << "Unknown ResolveHostParameters_DnsQueryType value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(ResolveHostParameters_DnsQueryType value) {
  return internal::ResolveHostParameters_DnsQueryType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class ResolveHostHandleInterfaceBase {};

using ResolveHostHandlePtrDataView =
    mojo::InterfacePtrDataView<ResolveHostHandleInterfaceBase>;
using ResolveHostHandleRequestDataView =
    mojo::InterfaceRequestDataView<ResolveHostHandleInterfaceBase>;
using ResolveHostHandleAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<ResolveHostHandleInterfaceBase>;
using ResolveHostHandleAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<ResolveHostHandleInterfaceBase>;
class ResolveHostClientInterfaceBase {};

using ResolveHostClientPtrDataView =
    mojo::InterfacePtrDataView<ResolveHostClientInterfaceBase>;
using ResolveHostClientRequestDataView =
    mojo::InterfaceRequestDataView<ResolveHostClientInterfaceBase>;
using ResolveHostClientAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<ResolveHostClientInterfaceBase>;
using ResolveHostClientAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<ResolveHostClientInterfaceBase>;
class HostResolverInterfaceBase {};

using HostResolverPtrDataView =
    mojo::InterfacePtrDataView<HostResolverInterfaceBase>;
using HostResolverRequestDataView =
    mojo::InterfaceRequestDataView<HostResolverInterfaceBase>;
using HostResolverAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<HostResolverInterfaceBase>;
using HostResolverAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<HostResolverInterfaceBase>;
class ResolveHostParametersDataView {
 public:
  ResolveHostParametersDataView() {}

  ResolveHostParametersDataView(
      internal::ResolveHostParameters_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDnsQueryType(UserType* output) const {
    auto data_value = data_->dns_query_type;
    return mojo::internal::Deserialize<::network::mojom::ResolveHostParameters_DnsQueryType>(
        data_value, output);
  }

  ResolveHostParameters_DnsQueryType dns_query_type() const {
    return static_cast<ResolveHostParameters_DnsQueryType>(data_->dns_query_type);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInitialPriority(UserType* output) const {
    auto data_value = data_->initial_priority;
    return mojo::internal::Deserialize<::network::mojom::RequestPriority>(
        data_value, output);
  }

  ::network::mojom::RequestPriority initial_priority() const {
    return static_cast<::network::mojom::RequestPriority>(data_->initial_priority);
  }
  template <typename UserType>
  UserType TakeControlHandle() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::ResolveHostHandleRequestDataView>(
            &data_->control_handle, &result, context_);
    DCHECK(ret);
    return result;
  }
  bool include_canonical_name() const {
    return data_->include_canonical_name;
  }
  bool loopback_only() const {
    return data_->loopback_only;
  }
  bool is_speculative() const {
    return data_->is_speculative;
  }
 private:
  internal::ResolveHostParameters_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ResolveHostHandle_Cancel_ParamsDataView {
 public:
  ResolveHostHandle_Cancel_ParamsDataView() {}

  ResolveHostHandle_Cancel_ParamsDataView(
      internal::ResolveHostHandle_Cancel_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t result() const {
    return data_->result;
  }
 private:
  internal::ResolveHostHandle_Cancel_Params_Data* data_ = nullptr;
};

class ResolveHostClient_OnComplete_ParamsDataView {
 public:
  ResolveHostClient_OnComplete_ParamsDataView() {}

  ResolveHostClient_OnComplete_ParamsDataView(
      internal::ResolveHostClient_OnComplete_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t result() const {
    return data_->result;
  }
  inline void GetResolvedAddressesDataView(
      ::net::interfaces::AddressListDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResolvedAddresses(UserType* output) {
    auto* pointer = data_->resolved_addresses.Get();
    return mojo::internal::Deserialize<::net::interfaces::AddressListDataView>(
        pointer, output, context_);
  }
 private:
  internal::ResolveHostClient_OnComplete_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class HostResolver_ResolveHost_ParamsDataView {
 public:
  HostResolver_ResolveHost_ParamsDataView() {}

  HostResolver_ResolveHost_ParamsDataView(
      internal::HostResolver_ResolveHost_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetHostDataView(
      ::network::mojom::HostPortPairDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHost(UserType* output) {
    auto* pointer = data_->host.Get();
    return mojo::internal::Deserialize<::network::mojom::HostPortPairDataView>(
        pointer, output, context_);
  }
  inline void GetOptionalParametersDataView(
      ResolveHostParametersDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOptionalParameters(UserType* output) {
    auto* pointer = data_->optional_parameters.Get();
    return mojo::internal::Deserialize<::network::mojom::ResolveHostParametersDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeResponseClient() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::ResolveHostClientPtrDataView>(
            &data_->response_client, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::HostResolver_ResolveHost_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace network

namespace std {

template <>
struct hash<::network::mojom::ResolveHostParameters_DnsQueryType>
    : public mojo::internal::EnumHashImpl<::network::mojom::ResolveHostParameters_DnsQueryType> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::network::mojom::ResolveHostParameters_DnsQueryType, ::network::mojom::ResolveHostParameters_DnsQueryType> {
  static ::network::mojom::ResolveHostParameters_DnsQueryType ToMojom(::network::mojom::ResolveHostParameters_DnsQueryType input) { return input; }
  static bool FromMojom(::network::mojom::ResolveHostParameters_DnsQueryType input, ::network::mojom::ResolveHostParameters_DnsQueryType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::network::mojom::ResolveHostParameters_DnsQueryType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::network::mojom::ResolveHostParameters_DnsQueryType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::network::mojom::ResolveHostParameters_DnsQueryType>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::network::mojom::ResolveHostParametersDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::network::mojom::ResolveHostParametersDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::network::mojom::internal::ResolveHostParameters_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    mojo::internal::Serialize<::network::mojom::ResolveHostParameters_DnsQueryType>(
        CallWithContext(Traits::dns_query_type, input, custom_context), &(*output)->dns_query_type);
    mojo::internal::Serialize<::network::mojom::RequestPriority>(
        CallWithContext(Traits::initial_priority, input, custom_context), &(*output)->initial_priority);
    decltype(CallWithContext(Traits::control_handle, input, custom_context)) in_control_handle = CallWithContext(Traits::control_handle, input, custom_context);
    mojo::internal::Serialize<::network::mojom::ResolveHostHandleRequestDataView>(
        in_control_handle, &(*output)->control_handle, context);
    (*output)->include_canonical_name = CallWithContext(Traits::include_canonical_name, input, custom_context);
    (*output)->loopback_only = CallWithContext(Traits::loopback_only, input, custom_context);
    (*output)->is_speculative = CallWithContext(Traits::is_speculative, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::network::mojom::internal::ResolveHostParameters_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::network::mojom::ResolveHostParametersDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace network {
namespace mojom {





inline void ResolveHostClient_OnComplete_ParamsDataView::GetResolvedAddressesDataView(
    ::net::interfaces::AddressListDataView* output) {
  auto pointer = data_->resolved_addresses.Get();
  *output = ::net::interfaces::AddressListDataView(pointer, context_);
}


inline void HostResolver_ResolveHost_ParamsDataView::GetHostDataView(
    ::network::mojom::HostPortPairDataView* output) {
  auto pointer = data_->host.Get();
  *output = ::network::mojom::HostPortPairDataView(pointer, context_);
}
inline void HostResolver_ResolveHost_ParamsDataView::GetOptionalParametersDataView(
    ResolveHostParametersDataView* output) {
  auto pointer = data_->optional_parameters.Get();
  *output = ResolveHostParametersDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace network

#endif  // SERVICES_NETWORK_PUBLIC_MOJOM_HOST_RESOLVER_MOJOM_SHARED_H_
