// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_NETWORK_PUBLIC_MOJOM_HOST_RESOLVER_MOJOM_H_
#define SERVICES_NETWORK_PUBLIC_MOJOM_HOST_RESOLVER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/network/public/mojom/host_resolver.mojom-shared.h"
#include "net/interfaces/address_list.mojom.h"
#include "services/network/public/mojom/network_param.mojom.h"
#include "services/network/public/mojom/url_loader.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "net/dns/host_resolver.h"


namespace network {
namespace mojom {
class ResolveHostHandle;
using ResolveHostHandlePtr = mojo::InterfacePtr<ResolveHostHandle>;
using ResolveHostHandlePtrInfo = mojo::InterfacePtrInfo<ResolveHostHandle>;
using ThreadSafeResolveHostHandlePtr =
    mojo::ThreadSafeInterfacePtr<ResolveHostHandle>;
using ResolveHostHandleRequest = mojo::InterfaceRequest<ResolveHostHandle>;
using ResolveHostHandleAssociatedPtr =
    mojo::AssociatedInterfacePtr<ResolveHostHandle>;
using ThreadSafeResolveHostHandleAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ResolveHostHandle>;
using ResolveHostHandleAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ResolveHostHandle>;
using ResolveHostHandleAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ResolveHostHandle>;

class ResolveHostClient;
using ResolveHostClientPtr = mojo::InterfacePtr<ResolveHostClient>;
using ResolveHostClientPtrInfo = mojo::InterfacePtrInfo<ResolveHostClient>;
using ThreadSafeResolveHostClientPtr =
    mojo::ThreadSafeInterfacePtr<ResolveHostClient>;
using ResolveHostClientRequest = mojo::InterfaceRequest<ResolveHostClient>;
using ResolveHostClientAssociatedPtr =
    mojo::AssociatedInterfacePtr<ResolveHostClient>;
using ThreadSafeResolveHostClientAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ResolveHostClient>;
using ResolveHostClientAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ResolveHostClient>;
using ResolveHostClientAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ResolveHostClient>;

class HostResolver;
using HostResolverPtr = mojo::InterfacePtr<HostResolver>;
using HostResolverPtrInfo = mojo::InterfacePtrInfo<HostResolver>;
using ThreadSafeHostResolverPtr =
    mojo::ThreadSafeInterfacePtr<HostResolver>;
using HostResolverRequest = mojo::InterfaceRequest<HostResolver>;
using HostResolverAssociatedPtr =
    mojo::AssociatedInterfacePtr<HostResolver>;
using ThreadSafeHostResolverAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<HostResolver>;
using HostResolverAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<HostResolver>;
using HostResolverAssociatedRequest =
    mojo::AssociatedInterfaceRequest<HostResolver>;

class ResolveHostParameters;
using ResolveHostParametersPtr = mojo::StructPtr<ResolveHostParameters>;


class ResolveHostHandleProxy;

template <typename ImplRefTraits>
class ResolveHostHandleStub;

class ResolveHostHandleRequestValidator;

class  ResolveHostHandle
    : public ResolveHostHandleInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ResolveHostHandleProxy;

  template <typename ImplRefTraits>
  using Stub_ = ResolveHostHandleStub<ImplRefTraits>;

  using RequestValidator_ = ResolveHostHandleRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kCancelMinVersion = 0,
  };
  virtual ~ResolveHostHandle() {}

  virtual void Cancel(int32_t result) = 0;
};
class  ResolveHostHandleInterceptorForTesting : public ResolveHostHandle {
  virtual ResolveHostHandle* GetForwardingInterface() = 0;
  void Cancel(int32_t result) override;
};
class  ResolveHostHandleAsyncWaiter {
 public:
  explicit ResolveHostHandleAsyncWaiter(ResolveHostHandle* proxy);
  ~ResolveHostHandleAsyncWaiter();

 private:
  ResolveHostHandle* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(ResolveHostHandleAsyncWaiter);
};

class ResolveHostClientProxy;

template <typename ImplRefTraits>
class ResolveHostClientStub;

class ResolveHostClientRequestValidator;

class  ResolveHostClient
    : public ResolveHostClientInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ResolveHostClientProxy;

  template <typename ImplRefTraits>
  using Stub_ = ResolveHostClientStub<ImplRefTraits>;

  using RequestValidator_ = ResolveHostClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnCompleteMinVersion = 0,
  };
  virtual ~ResolveHostClient() {}

  virtual void OnComplete(int32_t result, const base::Optional<net::AddressList>& resolved_addresses) = 0;
};
class  ResolveHostClientInterceptorForTesting : public ResolveHostClient {
  virtual ResolveHostClient* GetForwardingInterface() = 0;
  void OnComplete(int32_t result, const base::Optional<net::AddressList>& resolved_addresses) override;
};
class  ResolveHostClientAsyncWaiter {
 public:
  explicit ResolveHostClientAsyncWaiter(ResolveHostClient* proxy);
  ~ResolveHostClientAsyncWaiter();

 private:
  ResolveHostClient* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(ResolveHostClientAsyncWaiter);
};

class HostResolverProxy;

template <typename ImplRefTraits>
class HostResolverStub;

class HostResolverRequestValidator;

class  HostResolver
    : public HostResolverInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = HostResolverProxy;

  template <typename ImplRefTraits>
  using Stub_ = HostResolverStub<ImplRefTraits>;

  using RequestValidator_ = HostResolverRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kResolveHostMinVersion = 0,
  };
  virtual ~HostResolver() {}

  virtual void ResolveHost(const net::HostPortPair& host, ResolveHostParametersPtr optional_parameters, ResolveHostClientPtr response_client) = 0;
};
class  HostResolverInterceptorForTesting : public HostResolver {
  virtual HostResolver* GetForwardingInterface() = 0;
  void ResolveHost(const net::HostPortPair& host, ResolveHostParametersPtr optional_parameters, ResolveHostClientPtr response_client) override;
};
class  HostResolverAsyncWaiter {
 public:
  explicit HostResolverAsyncWaiter(HostResolver* proxy);
  ~HostResolverAsyncWaiter();

 private:
  HostResolver* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(HostResolverAsyncWaiter);
};

class  ResolveHostHandleProxy
    : public ResolveHostHandle {
 public:
  explicit ResolveHostHandleProxy(mojo::MessageReceiverWithResponder* receiver);
  void Cancel(int32_t result) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  ResolveHostClientProxy
    : public ResolveHostClient {
 public:
  explicit ResolveHostClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnComplete(int32_t result, const base::Optional<net::AddressList>& resolved_addresses) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  HostResolverProxy
    : public HostResolver {
 public:
  explicit HostResolverProxy(mojo::MessageReceiverWithResponder* receiver);
  void ResolveHost(const net::HostPortPair& host, ResolveHostParametersPtr optional_parameters, ResolveHostClientPtr response_client) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  ResolveHostHandleStubDispatch {
 public:
  static bool Accept(ResolveHostHandle* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ResolveHostHandle* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ResolveHostHandle>>
class ResolveHostHandleStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ResolveHostHandleStub() {}
  ~ResolveHostHandleStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ResolveHostHandleStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ResolveHostHandleStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  ResolveHostClientStubDispatch {
 public:
  static bool Accept(ResolveHostClient* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ResolveHostClient* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ResolveHostClient>>
class ResolveHostClientStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ResolveHostClientStub() {}
  ~ResolveHostClientStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ResolveHostClientStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ResolveHostClientStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  HostResolverStubDispatch {
 public:
  static bool Accept(HostResolver* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      HostResolver* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<HostResolver>>
class HostResolverStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  HostResolverStub() {}
  ~HostResolverStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return HostResolverStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return HostResolverStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  ResolveHostHandleRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  ResolveHostClientRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  HostResolverRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





class  ResolveHostParameters {
 public:
  using DataView = ResolveHostParametersDataView;
  using Data_ = internal::ResolveHostParameters_Data;using DnsQueryType = ResolveHostParameters_DnsQueryType;

  template <typename... Args>
  static ResolveHostParametersPtr New(Args&&... args) {
    return ResolveHostParametersPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static ResolveHostParametersPtr From(const U& u) {
    return mojo::TypeConverter<ResolveHostParametersPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ResolveHostParameters>::Convert(*this);
  }


  ResolveHostParameters();

  ResolveHostParameters(
      net::HostResolver::DnsQueryType dns_query_type,
      net::RequestPriority initial_priority,
      ResolveHostHandleRequest control_handle,
      bool include_canonical_name,
      bool loopback_only,
      bool is_speculative);

  ~ResolveHostParameters();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ResolveHostParametersPtr>
  ResolveHostParametersPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ResolveHostParameters>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        ResolveHostParameters::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        ResolveHostParameters::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::ResolveHostParameters_UnserializedMessageContext<
            UserType, ResolveHostParameters::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<ResolveHostParameters::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return ResolveHostParameters::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::ResolveHostParameters_UnserializedMessageContext<
            UserType, ResolveHostParameters::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<ResolveHostParameters::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  net::HostResolver::DnsQueryType dns_query_type;
  net::RequestPriority initial_priority;
  ResolveHostHandleRequest control_handle;
  bool include_canonical_name;
  bool loopback_only;
  bool is_speculative;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(ResolveHostParameters);
};

template <typename StructPtrType>
ResolveHostParametersPtr ResolveHostParameters::Clone() const {
  return New(
      mojo::Clone(dns_query_type),
      mojo::Clone(initial_priority),
      mojo::Clone(control_handle),
      mojo::Clone(include_canonical_name),
      mojo::Clone(loopback_only),
      mojo::Clone(is_speculative)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ResolveHostParameters>::value>::type*>
bool ResolveHostParameters::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->dns_query_type, other_struct.dns_query_type))
    return false;
  if (!mojo::Equals(this->initial_priority, other_struct.initial_priority))
    return false;
  if (!mojo::Equals(this->control_handle, other_struct.control_handle))
    return false;
  if (!mojo::Equals(this->include_canonical_name, other_struct.include_canonical_name))
    return false;
  if (!mojo::Equals(this->loopback_only, other_struct.loopback_only))
    return false;
  if (!mojo::Equals(this->is_speculative, other_struct.is_speculative))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace network

namespace mojo {


template <>
struct  StructTraits<::network::mojom::ResolveHostParameters::DataView,
                                         ::network::mojom::ResolveHostParametersPtr> {
  static bool IsNull(const ::network::mojom::ResolveHostParametersPtr& input) { return !input; }
  static void SetToNull(::network::mojom::ResolveHostParametersPtr* output) { output->reset(); }

  static decltype(::network::mojom::ResolveHostParameters::dns_query_type) dns_query_type(
      const ::network::mojom::ResolveHostParametersPtr& input) {
    return input->dns_query_type;
  }

  static decltype(::network::mojom::ResolveHostParameters::initial_priority) initial_priority(
      const ::network::mojom::ResolveHostParametersPtr& input) {
    return input->initial_priority;
  }

  static  decltype(::network::mojom::ResolveHostParameters::control_handle)& control_handle(
       ::network::mojom::ResolveHostParametersPtr& input) {
    return input->control_handle;
  }

  static decltype(::network::mojom::ResolveHostParameters::include_canonical_name) include_canonical_name(
      const ::network::mojom::ResolveHostParametersPtr& input) {
    return input->include_canonical_name;
  }

  static decltype(::network::mojom::ResolveHostParameters::loopback_only) loopback_only(
      const ::network::mojom::ResolveHostParametersPtr& input) {
    return input->loopback_only;
  }

  static decltype(::network::mojom::ResolveHostParameters::is_speculative) is_speculative(
      const ::network::mojom::ResolveHostParametersPtr& input) {
    return input->is_speculative;
  }

  static bool Read(::network::mojom::ResolveHostParameters::DataView input, ::network::mojom::ResolveHostParametersPtr* output);
};

}  // namespace mojo

#endif  // SERVICES_NETWORK_PUBLIC_MOJOM_HOST_RESOLVER_MOJOM_H_