// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "ui/accessibility/mojom/ax_host.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "ui/accessibility/mojom/ax_host.mojom-shared-message-ids.h"
#include "ui/accessibility/mojom/ax_action_data_mojom_traits.h"
#include "ui/accessibility/mojom/ax_event_mojom_traits.h"
#include "ui/accessibility/mojom/ax_node_data_mojom_traits.h"
#include "ui/accessibility/mojom/ax_tree_data_mojom_traits.h"
#include "ui/accessibility/mojom/ax_tree_update_mojom_traits.h"
#include "ui/gfx/geometry/mojo/geometry_struct_traits.h"
#include "ui/gfx/mojo/transform_struct_traits.h"
namespace ax {
namespace mojom {
const char kAXHostServiceName[] = "ax_host_service";
const char AXHost::Name_[] = "ax.mojom.AXHost";

AXHostProxy::AXHostProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void AXHostProxy::SetRemoteHost(
    AXRemoteHostPtr in_remote) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "ax::mojom::AXHost::SetRemoteHost");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kAXHost_SetRemoteHost_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::ax::mojom::internal::AXHost_SetRemoteHost_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::ax::mojom::AXRemoteHostPtrDataView>(
      in_remote, &params->remote, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->remote),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid remote in AXHost.SetRemoteHost request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(AXHost::Name_);
  message.set_method_name("SetRemoteHost");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void AXHostProxy::HandleAccessibilityEvent(
    int32_t in_tree_id, const std::vector<ui::AXTreeUpdate>& in_updates, const ui::AXEvent& in_event) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "ax::mojom::AXHost::HandleAccessibilityEvent");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kAXHost_HandleAccessibilityEvent_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::ax::mojom::internal::AXHost_HandleAccessibilityEvent_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->tree_id = in_tree_id;
  typename decltype(params->updates)::BaseType::BufferWriter
      updates_writer;
  const mojo::internal::ContainerValidateParams updates_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::ax::mojom::AXTreeUpdateDataView>>(
      in_updates, buffer, &updates_writer, &updates_validate_params,
      &serialization_context);
  params->updates.Set(
      updates_writer.is_null() ? nullptr : updates_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->updates.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null updates in AXHost.HandleAccessibilityEvent request");
  typename decltype(params->event)::BaseType::BufferWriter
      event_writer;
  mojo::internal::Serialize<::ax::mojom::AXEventDataView>(
      in_event, buffer, &event_writer, &serialization_context);
  params->event.Set(
      event_writer.is_null() ? nullptr : event_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->event.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null event in AXHost.HandleAccessibilityEvent request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(AXHost::Name_);
  message.set_method_name("HandleAccessibilityEvent");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool AXHostStubDispatch::Accept(
    AXHost* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kAXHost_SetRemoteHost_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)ax::mojom::AXHost::SetRemoteHost",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::AXHost_SetRemoteHost_Params_Data* params =
          reinterpret_cast<internal::AXHost_SetRemoteHost_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      AXRemoteHostPtr p_remote{};
      AXHost_SetRemoteHost_ParamsDataView input_data_view(params, &serialization_context);
      
      p_remote =
          input_data_view.TakeRemote<decltype(p_remote)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "AXHost::SetRemoteHost deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetRemoteHost(
std::move(p_remote));
      return true;
    }
    case internal::kAXHost_HandleAccessibilityEvent_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)ax::mojom::AXHost::HandleAccessibilityEvent",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::AXHost_HandleAccessibilityEvent_Params_Data* params =
          reinterpret_cast<internal::AXHost_HandleAccessibilityEvent_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int32_t p_tree_id{};
      std::vector<ui::AXTreeUpdate> p_updates{};
      ui::AXEvent p_event{};
      AXHost_HandleAccessibilityEvent_ParamsDataView input_data_view(params, &serialization_context);
      
      p_tree_id = input_data_view.tree_id();
      if (!input_data_view.ReadUpdates(&p_updates))
        success = false;
      if (!input_data_view.ReadEvent(&p_event))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "AXHost::HandleAccessibilityEvent deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->HandleAccessibilityEvent(
std::move(p_tree_id), 
std::move(p_updates), 
std::move(p_event));
      return true;
    }
  }
  return false;
}

// static
bool AXHostStubDispatch::AcceptWithResponder(
    AXHost* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kAXHost_SetRemoteHost_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)ax::mojom::AXHost::SetRemoteHost",
               "message", message->name());
#endif
      break;
    }
    case internal::kAXHost_HandleAccessibilityEvent_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)ax::mojom::AXHost::HandleAccessibilityEvent",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool AXHostRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "AXHost RequestValidator");

  switch (message->header()->name) {
    case internal::kAXHost_SetRemoteHost_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::AXHost_SetRemoteHost_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kAXHost_HandleAccessibilityEvent_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::AXHost_HandleAccessibilityEvent_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void AXHostInterceptorForTesting::SetRemoteHost(AXRemoteHostPtr remote) {
  GetForwardingInterface()->SetRemoteHost(std::move(remote));
}
void AXHostInterceptorForTesting::HandleAccessibilityEvent(int32_t tree_id, const std::vector<ui::AXTreeUpdate>& updates, const ui::AXEvent& event) {
  GetForwardingInterface()->HandleAccessibilityEvent(std::move(tree_id), std::move(updates), std::move(event));
}
AXHostAsyncWaiter::AXHostAsyncWaiter(
    AXHost* proxy) : proxy_(proxy) {}

AXHostAsyncWaiter::~AXHostAsyncWaiter() = default;


const char AXRemoteHost::Name_[] = "ax.mojom.AXRemoteHost";

AXRemoteHostProxy::AXRemoteHostProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void AXRemoteHostProxy::OnAutomationEnabled(
    bool in_enabled) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "ax::mojom::AXRemoteHost::OnAutomationEnabled");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kAXRemoteHost_OnAutomationEnabled_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::ax::mojom::internal::AXRemoteHost_OnAutomationEnabled_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->enabled = in_enabled;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(AXRemoteHost::Name_);
  message.set_method_name("OnAutomationEnabled");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void AXRemoteHostProxy::PerformAction(
    const ui::AXActionData& in_data) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "ax::mojom::AXRemoteHost::PerformAction");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kAXRemoteHost_PerformAction_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::ax::mojom::internal::AXRemoteHost_PerformAction_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->data)::BaseType::BufferWriter
      data_writer;
  mojo::internal::Serialize<::ax::mojom::AXActionDataDataView>(
      in_data, buffer, &data_writer, &serialization_context);
  params->data.Set(
      data_writer.is_null() ? nullptr : data_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->data.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null data in AXRemoteHost.PerformAction request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(AXRemoteHost::Name_);
  message.set_method_name("PerformAction");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool AXRemoteHostStubDispatch::Accept(
    AXRemoteHost* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kAXRemoteHost_OnAutomationEnabled_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)ax::mojom::AXRemoteHost::OnAutomationEnabled",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::AXRemoteHost_OnAutomationEnabled_Params_Data* params =
          reinterpret_cast<internal::AXRemoteHost_OnAutomationEnabled_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      bool p_enabled{};
      AXRemoteHost_OnAutomationEnabled_ParamsDataView input_data_view(params, &serialization_context);
      
      p_enabled = input_data_view.enabled();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "AXRemoteHost::OnAutomationEnabled deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnAutomationEnabled(
std::move(p_enabled));
      return true;
    }
    case internal::kAXRemoteHost_PerformAction_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)ax::mojom::AXRemoteHost::PerformAction",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::AXRemoteHost_PerformAction_Params_Data* params =
          reinterpret_cast<internal::AXRemoteHost_PerformAction_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ui::AXActionData p_data{};
      AXRemoteHost_PerformAction_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadData(&p_data))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "AXRemoteHost::PerformAction deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->PerformAction(
std::move(p_data));
      return true;
    }
  }
  return false;
}

// static
bool AXRemoteHostStubDispatch::AcceptWithResponder(
    AXRemoteHost* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kAXRemoteHost_OnAutomationEnabled_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)ax::mojom::AXRemoteHost::OnAutomationEnabled",
               "message", message->name());
#endif
      break;
    }
    case internal::kAXRemoteHost_PerformAction_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)ax::mojom::AXRemoteHost::PerformAction",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool AXRemoteHostRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "AXRemoteHost RequestValidator");

  switch (message->header()->name) {
    case internal::kAXRemoteHost_OnAutomationEnabled_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::AXRemoteHost_OnAutomationEnabled_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kAXRemoteHost_PerformAction_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::AXRemoteHost_PerformAction_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void AXRemoteHostInterceptorForTesting::OnAutomationEnabled(bool enabled) {
  GetForwardingInterface()->OnAutomationEnabled(std::move(enabled));
}
void AXRemoteHostInterceptorForTesting::PerformAction(const ui::AXActionData& data) {
  GetForwardingInterface()->PerformAction(std::move(data));
}
AXRemoteHostAsyncWaiter::AXRemoteHostAsyncWaiter(
    AXRemoteHost* proxy) : proxy_(proxy) {}

AXRemoteHostAsyncWaiter::~AXRemoteHostAsyncWaiter() = default;


}  // namespace mojom
}  // namespace ax

namespace mojo {

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif