// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_VIEWS_BRIDGE_MAC_MOJO_BRIDGED_NATIVE_WIDGET_MOJOM_SHARED_H_
#define UI_VIEWS_BRIDGE_MAC_MOJO_BRIDGED_NATIVE_WIDGET_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "ui/views_bridge_mac/mojo/bridged_native_widget.mojom-shared-internal.h"
#include "mojo/public/mojom/base/string16.mojom-shared.h"
#include "ui/base/mojo/ui_base_types.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"
#include "ui/gfx/mojo/ca_layer_params.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace views_bridge_mac {
namespace mojom {
class BridgedNativeWidgetInitParamsDataView;



}  // namespace mojom
}  // namespace views_bridge_mac

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::views_bridge_mac::mojom::BridgedNativeWidgetInitParamsDataView> {
  using Data = ::views_bridge_mac::mojom::internal::BridgedNativeWidgetInitParams_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace views_bridge_mac {
namespace mojom {

enum class WindowVisibilityState : int32_t {
  kHideWindow,
  kShowAndActivateWindow,
  kShowInactive,
  kMinValue = 0,
  kMaxValue = 2,
};

inline std::ostream& operator<<(std::ostream& os, WindowVisibilityState value) {
  switch(value) {
    case WindowVisibilityState::kHideWindow:
      return os << "WindowVisibilityState::kHideWindow";
    case WindowVisibilityState::kShowAndActivateWindow:
      return os << "WindowVisibilityState::kShowAndActivateWindow";
    case WindowVisibilityState::kShowInactive:
      return os << "WindowVisibilityState::kShowInactive";
    default:
      return os << "Unknown WindowVisibilityState value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(WindowVisibilityState value) {
  return internal::WindowVisibilityState_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class BridgedNativeWidgetInterfaceBase {};

using BridgedNativeWidgetPtrDataView =
    mojo::InterfacePtrDataView<BridgedNativeWidgetInterfaceBase>;
using BridgedNativeWidgetRequestDataView =
    mojo::InterfaceRequestDataView<BridgedNativeWidgetInterfaceBase>;
using BridgedNativeWidgetAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<BridgedNativeWidgetInterfaceBase>;
using BridgedNativeWidgetAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<BridgedNativeWidgetInterfaceBase>;
class BridgedNativeWidgetInitParamsDataView {
 public:
  BridgedNativeWidgetInitParamsDataView() {}

  BridgedNativeWidgetInitParamsDataView(
      internal::BridgedNativeWidgetInitParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadModalType(UserType* output) const {
    auto data_value = data_->modal_type;
    return mojo::internal::Deserialize<::ui::mojom::ModalType>(
        data_value, output);
  }

  ::ui::mojom::ModalType modal_type() const {
    return static_cast<::ui::mojom::ModalType>(data_->modal_type);
  }
  bool is_translucent() const {
    return data_->is_translucent;
  }
  bool has_window_server_shadow() const {
    return data_->has_window_server_shadow;
  }
  bool force_into_collection_cycle() const {
    return data_->force_into_collection_cycle;
  }
 private:
  internal::BridgedNativeWidgetInitParams_Data* data_ = nullptr;
};

class BridgedNativeWidget_InitWindow_ParamsDataView {
 public:
  BridgedNativeWidget_InitWindow_ParamsDataView() {}

  BridgedNativeWidget_InitWindow_ParamsDataView(
      internal::BridgedNativeWidget_InitWindow_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetParamsDataView(
      BridgedNativeWidgetInitParamsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadParams(UserType* output) {
    auto* pointer = data_->params.Get();
    return mojo::internal::Deserialize<::views_bridge_mac::mojom::BridgedNativeWidgetInitParamsDataView>(
        pointer, output, context_);
  }
 private:
  internal::BridgedNativeWidget_InitWindow_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BridgedNativeWidget_InitCompositorView_ParamsDataView {
 public:
  BridgedNativeWidget_InitCompositorView_ParamsDataView() {}

  BridgedNativeWidget_InitCompositorView_ParamsDataView(
      internal::BridgedNativeWidget_InitCompositorView_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::BridgedNativeWidget_InitCompositorView_Params_Data* data_ = nullptr;
};

class BridgedNativeWidget_CreateContentView_ParamsDataView {
 public:
  BridgedNativeWidget_CreateContentView_ParamsDataView() {}

  BridgedNativeWidget_CreateContentView_ParamsDataView(
      internal::BridgedNativeWidget_CreateContentView_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetBoundsDataView(
      ::gfx::mojom::RectDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadBounds(UserType* output) {
    auto* pointer = data_->bounds.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectDataView>(
        pointer, output, context_);
  }
 private:
  internal::BridgedNativeWidget_CreateContentView_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BridgedNativeWidget_DestroyContentView_ParamsDataView {
 public:
  BridgedNativeWidget_DestroyContentView_ParamsDataView() {}

  BridgedNativeWidget_DestroyContentView_ParamsDataView(
      internal::BridgedNativeWidget_DestroyContentView_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::BridgedNativeWidget_DestroyContentView_Params_Data* data_ = nullptr;
};

class BridgedNativeWidget_CloseWindow_ParamsDataView {
 public:
  BridgedNativeWidget_CloseWindow_ParamsDataView() {}

  BridgedNativeWidget_CloseWindow_ParamsDataView(
      internal::BridgedNativeWidget_CloseWindow_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::BridgedNativeWidget_CloseWindow_Params_Data* data_ = nullptr;
};

class BridgedNativeWidget_CloseWindowNow_ParamsDataView {
 public:
  BridgedNativeWidget_CloseWindowNow_ParamsDataView() {}

  BridgedNativeWidget_CloseWindowNow_ParamsDataView(
      internal::BridgedNativeWidget_CloseWindowNow_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::BridgedNativeWidget_CloseWindowNow_Params_Data* data_ = nullptr;
};

class BridgedNativeWidget_SetInitialBounds_ParamsDataView {
 public:
  BridgedNativeWidget_SetInitialBounds_ParamsDataView() {}

  BridgedNativeWidget_SetInitialBounds_ParamsDataView(
      internal::BridgedNativeWidget_SetInitialBounds_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetNewBoundsDataView(
      ::gfx::mojom::RectDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadNewBounds(UserType* output) {
    auto* pointer = data_->new_bounds.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectDataView>(
        pointer, output, context_);
  }
  inline void GetMinimumContentSizeDataView(
      ::gfx::mojom::SizeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMinimumContentSize(UserType* output) {
    auto* pointer = data_->minimum_content_size.Get();
    return mojo::internal::Deserialize<::gfx::mojom::SizeDataView>(
        pointer, output, context_);
  }
  inline void GetParentOffsetDataView(
      ::gfx::mojom::Vector2dDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadParentOffset(UserType* output) {
    auto* pointer = data_->parent_offset.Get();
    return mojo::internal::Deserialize<::gfx::mojom::Vector2dDataView>(
        pointer, output, context_);
  }
 private:
  internal::BridgedNativeWidget_SetInitialBounds_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BridgedNativeWidget_SetBounds_ParamsDataView {
 public:
  BridgedNativeWidget_SetBounds_ParamsDataView() {}

  BridgedNativeWidget_SetBounds_ParamsDataView(
      internal::BridgedNativeWidget_SetBounds_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetNewBoundsDataView(
      ::gfx::mojom::RectDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadNewBounds(UserType* output) {
    auto* pointer = data_->new_bounds.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectDataView>(
        pointer, output, context_);
  }
  inline void GetMinimumContentSizeDataView(
      ::gfx::mojom::SizeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMinimumContentSize(UserType* output) {
    auto* pointer = data_->minimum_content_size.Get();
    return mojo::internal::Deserialize<::gfx::mojom::SizeDataView>(
        pointer, output, context_);
  }
 private:
  internal::BridgedNativeWidget_SetBounds_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BridgedNativeWidget_SetVisibilityState_ParamsDataView {
 public:
  BridgedNativeWidget_SetVisibilityState_ParamsDataView() {}

  BridgedNativeWidget_SetVisibilityState_ParamsDataView(
      internal::BridgedNativeWidget_SetVisibilityState_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadNewState(UserType* output) const {
    auto data_value = data_->new_state;
    return mojo::internal::Deserialize<::views_bridge_mac::mojom::WindowVisibilityState>(
        data_value, output);
  }

  WindowVisibilityState new_state() const {
    return static_cast<WindowVisibilityState>(data_->new_state);
  }
 private:
  internal::BridgedNativeWidget_SetVisibilityState_Params_Data* data_ = nullptr;
};

class BridgedNativeWidget_SetVisibleOnAllSpaces_ParamsDataView {
 public:
  BridgedNativeWidget_SetVisibleOnAllSpaces_ParamsDataView() {}

  BridgedNativeWidget_SetVisibleOnAllSpaces_ParamsDataView(
      internal::BridgedNativeWidget_SetVisibleOnAllSpaces_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool always_visible() const {
    return data_->always_visible;
  }
 private:
  internal::BridgedNativeWidget_SetVisibleOnAllSpaces_Params_Data* data_ = nullptr;
};

class BridgedNativeWidget_SetFullscreen_ParamsDataView {
 public:
  BridgedNativeWidget_SetFullscreen_ParamsDataView() {}

  BridgedNativeWidget_SetFullscreen_ParamsDataView(
      internal::BridgedNativeWidget_SetFullscreen_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool fullscreen() const {
    return data_->fullscreen;
  }
 private:
  internal::BridgedNativeWidget_SetFullscreen_Params_Data* data_ = nullptr;
};

class BridgedNativeWidget_SetMiniaturized_ParamsDataView {
 public:
  BridgedNativeWidget_SetMiniaturized_ParamsDataView() {}

  BridgedNativeWidget_SetMiniaturized_ParamsDataView(
      internal::BridgedNativeWidget_SetMiniaturized_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool miniaturized() const {
    return data_->miniaturized;
  }
 private:
  internal::BridgedNativeWidget_SetMiniaturized_Params_Data* data_ = nullptr;
};

class BridgedNativeWidget_SetSizeConstraints_ParamsDataView {
 public:
  BridgedNativeWidget_SetSizeConstraints_ParamsDataView() {}

  BridgedNativeWidget_SetSizeConstraints_ParamsDataView(
      internal::BridgedNativeWidget_SetSizeConstraints_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetMinSizeDataView(
      ::gfx::mojom::SizeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMinSize(UserType* output) {
    auto* pointer = data_->min_size.Get();
    return mojo::internal::Deserialize<::gfx::mojom::SizeDataView>(
        pointer, output, context_);
  }
  inline void GetMaxSizeDataView(
      ::gfx::mojom::SizeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMaxSize(UserType* output) {
    auto* pointer = data_->max_size.Get();
    return mojo::internal::Deserialize<::gfx::mojom::SizeDataView>(
        pointer, output, context_);
  }
  bool is_resizable() const {
    return data_->is_resizable;
  }
  bool is_maximizable() const {
    return data_->is_maximizable;
  }
 private:
  internal::BridgedNativeWidget_SetSizeConstraints_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BridgedNativeWidget_SetOpacity_ParamsDataView {
 public:
  BridgedNativeWidget_SetOpacity_ParamsDataView() {}

  BridgedNativeWidget_SetOpacity_ParamsDataView(
      internal::BridgedNativeWidget_SetOpacity_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  float opacity() const {
    return data_->opacity;
  }
 private:
  internal::BridgedNativeWidget_SetOpacity_Params_Data* data_ = nullptr;
};

class BridgedNativeWidget_SetContentAspectRatio_ParamsDataView {
 public:
  BridgedNativeWidget_SetContentAspectRatio_ParamsDataView() {}

  BridgedNativeWidget_SetContentAspectRatio_ParamsDataView(
      internal::BridgedNativeWidget_SetContentAspectRatio_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetAspectRatioDataView(
      ::gfx::mojom::SizeFDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAspectRatio(UserType* output) {
    auto* pointer = data_->aspect_ratio.Get();
    return mojo::internal::Deserialize<::gfx::mojom::SizeFDataView>(
        pointer, output, context_);
  }
 private:
  internal::BridgedNativeWidget_SetContentAspectRatio_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BridgedNativeWidget_SetCALayerParams_ParamsDataView {
 public:
  BridgedNativeWidget_SetCALayerParams_ParamsDataView() {}

  BridgedNativeWidget_SetCALayerParams_ParamsDataView(
      internal::BridgedNativeWidget_SetCALayerParams_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetCaLayerParamsDataView(
      ::gfx::mojom::CALayerParamsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCaLayerParams(UserType* output) {
    auto* pointer = data_->ca_layer_params.Get();
    return mojo::internal::Deserialize<::gfx::mojom::CALayerParamsDataView>(
        pointer, output, context_);
  }
 private:
  internal::BridgedNativeWidget_SetCALayerParams_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BridgedNativeWidget_SetWindowTitle_ParamsDataView {
 public:
  BridgedNativeWidget_SetWindowTitle_ParamsDataView() {}

  BridgedNativeWidget_SetWindowTitle_ParamsDataView(
      internal::BridgedNativeWidget_SetWindowTitle_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetTitleDataView(
      ::mojo_base::mojom::String16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTitle(UserType* output) {
    auto* pointer = data_->title.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::String16DataView>(
        pointer, output, context_);
  }
 private:
  internal::BridgedNativeWidget_SetWindowTitle_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BridgedNativeWidget_MakeFirstResponder_ParamsDataView {
 public:
  BridgedNativeWidget_MakeFirstResponder_ParamsDataView() {}

  BridgedNativeWidget_MakeFirstResponder_ParamsDataView(
      internal::BridgedNativeWidget_MakeFirstResponder_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::BridgedNativeWidget_MakeFirstResponder_Params_Data* data_ = nullptr;
};

class BridgedNativeWidget_ClearTouchBar_ParamsDataView {
 public:
  BridgedNativeWidget_ClearTouchBar_ParamsDataView() {}

  BridgedNativeWidget_ClearTouchBar_ParamsDataView(
      internal::BridgedNativeWidget_ClearTouchBar_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::BridgedNativeWidget_ClearTouchBar_Params_Data* data_ = nullptr;
};

class BridgedNativeWidget_AcquireCapture_ParamsDataView {
 public:
  BridgedNativeWidget_AcquireCapture_ParamsDataView() {}

  BridgedNativeWidget_AcquireCapture_ParamsDataView(
      internal::BridgedNativeWidget_AcquireCapture_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::BridgedNativeWidget_AcquireCapture_Params_Data* data_ = nullptr;
};

class BridgedNativeWidget_ReleaseCapture_ParamsDataView {
 public:
  BridgedNativeWidget_ReleaseCapture_ParamsDataView() {}

  BridgedNativeWidget_ReleaseCapture_ParamsDataView(
      internal::BridgedNativeWidget_ReleaseCapture_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::BridgedNativeWidget_ReleaseCapture_Params_Data* data_ = nullptr;
};



}  // namespace mojom
}  // namespace views_bridge_mac

namespace std {

template <>
struct hash<::views_bridge_mac::mojom::WindowVisibilityState>
    : public mojo::internal::EnumHashImpl<::views_bridge_mac::mojom::WindowVisibilityState> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::views_bridge_mac::mojom::WindowVisibilityState, ::views_bridge_mac::mojom::WindowVisibilityState> {
  static ::views_bridge_mac::mojom::WindowVisibilityState ToMojom(::views_bridge_mac::mojom::WindowVisibilityState input) { return input; }
  static bool FromMojom(::views_bridge_mac::mojom::WindowVisibilityState input, ::views_bridge_mac::mojom::WindowVisibilityState* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::views_bridge_mac::mojom::WindowVisibilityState, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::views_bridge_mac::mojom::WindowVisibilityState, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::views_bridge_mac::mojom::WindowVisibilityState>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::views_bridge_mac::mojom::BridgedNativeWidgetInitParamsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::views_bridge_mac::mojom::BridgedNativeWidgetInitParamsDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::views_bridge_mac::mojom::internal::BridgedNativeWidgetInitParams_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    mojo::internal::Serialize<::ui::mojom::ModalType>(
        CallWithContext(Traits::modal_type, input, custom_context), &(*output)->modal_type);
    (*output)->is_translucent = CallWithContext(Traits::is_translucent, input, custom_context);
    (*output)->has_window_server_shadow = CallWithContext(Traits::has_window_server_shadow, input, custom_context);
    (*output)->force_into_collection_cycle = CallWithContext(Traits::force_into_collection_cycle, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::views_bridge_mac::mojom::internal::BridgedNativeWidgetInitParams_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::views_bridge_mac::mojom::BridgedNativeWidgetInitParamsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace views_bridge_mac {
namespace mojom {



inline void BridgedNativeWidget_InitWindow_ParamsDataView::GetParamsDataView(
    BridgedNativeWidgetInitParamsDataView* output) {
  auto pointer = data_->params.Get();
  *output = BridgedNativeWidgetInitParamsDataView(pointer, context_);
}




inline void BridgedNativeWidget_CreateContentView_ParamsDataView::GetBoundsDataView(
    ::gfx::mojom::RectDataView* output) {
  auto pointer = data_->bounds.Get();
  *output = ::gfx::mojom::RectDataView(pointer, context_);
}








inline void BridgedNativeWidget_SetInitialBounds_ParamsDataView::GetNewBoundsDataView(
    ::gfx::mojom::RectDataView* output) {
  auto pointer = data_->new_bounds.Get();
  *output = ::gfx::mojom::RectDataView(pointer, context_);
}
inline void BridgedNativeWidget_SetInitialBounds_ParamsDataView::GetMinimumContentSizeDataView(
    ::gfx::mojom::SizeDataView* output) {
  auto pointer = data_->minimum_content_size.Get();
  *output = ::gfx::mojom::SizeDataView(pointer, context_);
}
inline void BridgedNativeWidget_SetInitialBounds_ParamsDataView::GetParentOffsetDataView(
    ::gfx::mojom::Vector2dDataView* output) {
  auto pointer = data_->parent_offset.Get();
  *output = ::gfx::mojom::Vector2dDataView(pointer, context_);
}


inline void BridgedNativeWidget_SetBounds_ParamsDataView::GetNewBoundsDataView(
    ::gfx::mojom::RectDataView* output) {
  auto pointer = data_->new_bounds.Get();
  *output = ::gfx::mojom::RectDataView(pointer, context_);
}
inline void BridgedNativeWidget_SetBounds_ParamsDataView::GetMinimumContentSizeDataView(
    ::gfx::mojom::SizeDataView* output) {
  auto pointer = data_->minimum_content_size.Get();
  *output = ::gfx::mojom::SizeDataView(pointer, context_);
}










inline void BridgedNativeWidget_SetSizeConstraints_ParamsDataView::GetMinSizeDataView(
    ::gfx::mojom::SizeDataView* output) {
  auto pointer = data_->min_size.Get();
  *output = ::gfx::mojom::SizeDataView(pointer, context_);
}
inline void BridgedNativeWidget_SetSizeConstraints_ParamsDataView::GetMaxSizeDataView(
    ::gfx::mojom::SizeDataView* output) {
  auto pointer = data_->max_size.Get();
  *output = ::gfx::mojom::SizeDataView(pointer, context_);
}




inline void BridgedNativeWidget_SetContentAspectRatio_ParamsDataView::GetAspectRatioDataView(
    ::gfx::mojom::SizeFDataView* output) {
  auto pointer = data_->aspect_ratio.Get();
  *output = ::gfx::mojom::SizeFDataView(pointer, context_);
}


inline void BridgedNativeWidget_SetCALayerParams_ParamsDataView::GetCaLayerParamsDataView(
    ::gfx::mojom::CALayerParamsDataView* output) {
  auto pointer = data_->ca_layer_params.Get();
  *output = ::gfx::mojom::CALayerParamsDataView(pointer, context_);
}


inline void BridgedNativeWidget_SetWindowTitle_ParamsDataView::GetTitleDataView(
    ::mojo_base::mojom::String16DataView* output) {
  auto pointer = data_->title.Get();
  *output = ::mojo_base::mojom::String16DataView(pointer, context_);
}











}  // namespace mojom
}  // namespace views_bridge_mac

#endif  // UI_VIEWS_BRIDGE_MAC_MOJO_BRIDGED_NATIVE_WIDGET_MOJOM_SHARED_H_
