// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_VIEWS_BRIDGE_MAC_MOJO_BRIDGED_NATIVE_WIDGET_MOJOM_H_
#define UI_VIEWS_BRIDGE_MAC_MOJO_BRIDGED_NATIVE_WIDGET_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "ui/views_bridge_mac/mojo/bridged_native_widget.mojom-shared.h"
#include "mojo/public/mojom/base/string16.mojom.h"
#include "ui/base/mojo/ui_base_types.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include "ui/gfx/mojo/ca_layer_params.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace views_bridge_mac {
namespace mojom {
class BridgedNativeWidget;
using BridgedNativeWidgetPtr = mojo::InterfacePtr<BridgedNativeWidget>;
using BridgedNativeWidgetPtrInfo = mojo::InterfacePtrInfo<BridgedNativeWidget>;
using ThreadSafeBridgedNativeWidgetPtr =
    mojo::ThreadSafeInterfacePtr<BridgedNativeWidget>;
using BridgedNativeWidgetRequest = mojo::InterfaceRequest<BridgedNativeWidget>;
using BridgedNativeWidgetAssociatedPtr =
    mojo::AssociatedInterfacePtr<BridgedNativeWidget>;
using ThreadSafeBridgedNativeWidgetAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<BridgedNativeWidget>;
using BridgedNativeWidgetAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<BridgedNativeWidget>;
using BridgedNativeWidgetAssociatedRequest =
    mojo::AssociatedInterfaceRequest<BridgedNativeWidget>;

class BridgedNativeWidgetInitParams;
using BridgedNativeWidgetInitParamsPtr = mojo::InlinedStructPtr<BridgedNativeWidgetInitParams>;


class BridgedNativeWidgetProxy;

template <typename ImplRefTraits>
class BridgedNativeWidgetStub;

class BridgedNativeWidgetRequestValidator;

class  BridgedNativeWidget
    : public BridgedNativeWidgetInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = BridgedNativeWidgetProxy;

  template <typename ImplRefTraits>
  using Stub_ = BridgedNativeWidgetStub<ImplRefTraits>;

  using RequestValidator_ = BridgedNativeWidgetRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kInitWindowMinVersion = 0,
    kInitCompositorViewMinVersion = 0,
    kCreateContentViewMinVersion = 0,
    kDestroyContentViewMinVersion = 0,
    kCloseWindowMinVersion = 0,
    kCloseWindowNowMinVersion = 0,
    kSetInitialBoundsMinVersion = 0,
    kSetBoundsMinVersion = 0,
    kSetVisibilityStateMinVersion = 0,
    kSetVisibleOnAllSpacesMinVersion = 0,
    kSetFullscreenMinVersion = 0,
    kSetMiniaturizedMinVersion = 0,
    kSetSizeConstraintsMinVersion = 0,
    kSetOpacityMinVersion = 0,
    kSetContentAspectRatioMinVersion = 0,
    kSetCALayerParamsMinVersion = 0,
    kSetWindowTitleMinVersion = 0,
    kMakeFirstResponderMinVersion = 0,
    kClearTouchBarMinVersion = 0,
    kAcquireCaptureMinVersion = 0,
    kReleaseCaptureMinVersion = 0,
  };
  virtual ~BridgedNativeWidget() {}

  virtual void InitWindow(BridgedNativeWidgetInitParamsPtr params) = 0;

  virtual void InitCompositorView() = 0;

  virtual void CreateContentView(const gfx::Rect& bounds) = 0;

  virtual void DestroyContentView() = 0;

  virtual void CloseWindow() = 0;

  virtual void CloseWindowNow() = 0;

  virtual void SetInitialBounds(const gfx::Rect& new_bounds, const gfx::Size& minimum_content_size, const gfx::Vector2d& parent_offset) = 0;

  virtual void SetBounds(const gfx::Rect& new_bounds, const gfx::Size& minimum_content_size) = 0;

  virtual void SetVisibilityState(WindowVisibilityState new_state) = 0;

  virtual void SetVisibleOnAllSpaces(bool always_visible) = 0;

  virtual void SetFullscreen(bool fullscreen) = 0;

  virtual void SetMiniaturized(bool miniaturized) = 0;

  virtual void SetSizeConstraints(const gfx::Size& min_size, const gfx::Size& max_size, bool is_resizable, bool is_maximizable) = 0;

  virtual void SetOpacity(float opacity) = 0;

  virtual void SetContentAspectRatio(const gfx::SizeF& aspect_ratio) = 0;

  virtual void SetCALayerParams(const gfx::CALayerParams& ca_layer_params) = 0;

  virtual void SetWindowTitle(const base::string16& title) = 0;

  virtual void MakeFirstResponder() = 0;

  virtual void ClearTouchBar() = 0;

  virtual void AcquireCapture() = 0;

  virtual void ReleaseCapture() = 0;
};
class  BridgedNativeWidgetInterceptorForTesting : public BridgedNativeWidget {
  virtual BridgedNativeWidget* GetForwardingInterface() = 0;
  void InitWindow(BridgedNativeWidgetInitParamsPtr params) override;
  void InitCompositorView() override;
  void CreateContentView(const gfx::Rect& bounds) override;
  void DestroyContentView() override;
  void CloseWindow() override;
  void CloseWindowNow() override;
  void SetInitialBounds(const gfx::Rect& new_bounds, const gfx::Size& minimum_content_size, const gfx::Vector2d& parent_offset) override;
  void SetBounds(const gfx::Rect& new_bounds, const gfx::Size& minimum_content_size) override;
  void SetVisibilityState(WindowVisibilityState new_state) override;
  void SetVisibleOnAllSpaces(bool always_visible) override;
  void SetFullscreen(bool fullscreen) override;
  void SetMiniaturized(bool miniaturized) override;
  void SetSizeConstraints(const gfx::Size& min_size, const gfx::Size& max_size, bool is_resizable, bool is_maximizable) override;
  void SetOpacity(float opacity) override;
  void SetContentAspectRatio(const gfx::SizeF& aspect_ratio) override;
  void SetCALayerParams(const gfx::CALayerParams& ca_layer_params) override;
  void SetWindowTitle(const base::string16& title) override;
  void MakeFirstResponder() override;
  void ClearTouchBar() override;
  void AcquireCapture() override;
  void ReleaseCapture() override;
};
class  BridgedNativeWidgetAsyncWaiter {
 public:
  explicit BridgedNativeWidgetAsyncWaiter(BridgedNativeWidget* proxy);
  ~BridgedNativeWidgetAsyncWaiter();

 private:
  BridgedNativeWidget* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(BridgedNativeWidgetAsyncWaiter);
};

class  BridgedNativeWidgetProxy
    : public BridgedNativeWidget {
 public:
  explicit BridgedNativeWidgetProxy(mojo::MessageReceiverWithResponder* receiver);
  void InitWindow(BridgedNativeWidgetInitParamsPtr params) final;
  void InitCompositorView() final;
  void CreateContentView(const gfx::Rect& bounds) final;
  void DestroyContentView() final;
  void CloseWindow() final;
  void CloseWindowNow() final;
  void SetInitialBounds(const gfx::Rect& new_bounds, const gfx::Size& minimum_content_size, const gfx::Vector2d& parent_offset) final;
  void SetBounds(const gfx::Rect& new_bounds, const gfx::Size& minimum_content_size) final;
  void SetVisibilityState(WindowVisibilityState new_state) final;
  void SetVisibleOnAllSpaces(bool always_visible) final;
  void SetFullscreen(bool fullscreen) final;
  void SetMiniaturized(bool miniaturized) final;
  void SetSizeConstraints(const gfx::Size& min_size, const gfx::Size& max_size, bool is_resizable, bool is_maximizable) final;
  void SetOpacity(float opacity) final;
  void SetContentAspectRatio(const gfx::SizeF& aspect_ratio) final;
  void SetCALayerParams(const gfx::CALayerParams& ca_layer_params) final;
  void SetWindowTitle(const base::string16& title) final;
  void MakeFirstResponder() final;
  void ClearTouchBar() final;
  void AcquireCapture() final;
  void ReleaseCapture() final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  BridgedNativeWidgetStubDispatch {
 public:
  static bool Accept(BridgedNativeWidget* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      BridgedNativeWidget* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<BridgedNativeWidget>>
class BridgedNativeWidgetStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  BridgedNativeWidgetStub() {}
  ~BridgedNativeWidgetStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BridgedNativeWidgetStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BridgedNativeWidgetStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  BridgedNativeWidgetRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};


class  BridgedNativeWidgetInitParams {
 public:
  using DataView = BridgedNativeWidgetInitParamsDataView;
  using Data_ = internal::BridgedNativeWidgetInitParams_Data;

  template <typename... Args>
  static BridgedNativeWidgetInitParamsPtr New(Args&&... args) {
    return BridgedNativeWidgetInitParamsPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static BridgedNativeWidgetInitParamsPtr From(const U& u) {
    return mojo::TypeConverter<BridgedNativeWidgetInitParamsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, BridgedNativeWidgetInitParams>::Convert(*this);
  }


  BridgedNativeWidgetInitParams();

  BridgedNativeWidgetInitParams(
      ui::ModalType modal_type,
      bool is_translucent,
      bool has_window_server_shadow,
      bool force_into_collection_cycle);

  ~BridgedNativeWidgetInitParams();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = BridgedNativeWidgetInitParamsPtr>
  BridgedNativeWidgetInitParamsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, BridgedNativeWidgetInitParams>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        BridgedNativeWidgetInitParams::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        BridgedNativeWidgetInitParams::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::BridgedNativeWidgetInitParams_UnserializedMessageContext<
            UserType, BridgedNativeWidgetInitParams::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<BridgedNativeWidgetInitParams::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return BridgedNativeWidgetInitParams::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::BridgedNativeWidgetInitParams_UnserializedMessageContext<
            UserType, BridgedNativeWidgetInitParams::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<BridgedNativeWidgetInitParams::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  ui::ModalType modal_type;
  bool is_translucent;
  bool has_window_server_shadow;
  bool force_into_collection_cycle;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};




template <typename StructPtrType>
BridgedNativeWidgetInitParamsPtr BridgedNativeWidgetInitParams::Clone() const {
  return New(
      mojo::Clone(modal_type),
      mojo::Clone(is_translucent),
      mojo::Clone(has_window_server_shadow),
      mojo::Clone(force_into_collection_cycle)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, BridgedNativeWidgetInitParams>::value>::type*>
bool BridgedNativeWidgetInitParams::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->modal_type, other_struct.modal_type))
    return false;
  if (!mojo::Equals(this->is_translucent, other_struct.is_translucent))
    return false;
  if (!mojo::Equals(this->has_window_server_shadow, other_struct.has_window_server_shadow))
    return false;
  if (!mojo::Equals(this->force_into_collection_cycle, other_struct.force_into_collection_cycle))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace views_bridge_mac

namespace mojo {


template <>
struct  StructTraits<::views_bridge_mac::mojom::BridgedNativeWidgetInitParams::DataView,
                                         ::views_bridge_mac::mojom::BridgedNativeWidgetInitParamsPtr> {
  static bool IsNull(const ::views_bridge_mac::mojom::BridgedNativeWidgetInitParamsPtr& input) { return !input; }
  static void SetToNull(::views_bridge_mac::mojom::BridgedNativeWidgetInitParamsPtr* output) { output->reset(); }

  static decltype(::views_bridge_mac::mojom::BridgedNativeWidgetInitParams::modal_type) modal_type(
      const ::views_bridge_mac::mojom::BridgedNativeWidgetInitParamsPtr& input) {
    return input->modal_type;
  }

  static decltype(::views_bridge_mac::mojom::BridgedNativeWidgetInitParams::is_translucent) is_translucent(
      const ::views_bridge_mac::mojom::BridgedNativeWidgetInitParamsPtr& input) {
    return input->is_translucent;
  }

  static decltype(::views_bridge_mac::mojom::BridgedNativeWidgetInitParams::has_window_server_shadow) has_window_server_shadow(
      const ::views_bridge_mac::mojom::BridgedNativeWidgetInitParamsPtr& input) {
    return input->has_window_server_shadow;
  }

  static decltype(::views_bridge_mac::mojom::BridgedNativeWidgetInitParams::force_into_collection_cycle) force_into_collection_cycle(
      const ::views_bridge_mac::mojom::BridgedNativeWidgetInitParamsPtr& input) {
    return input->force_into_collection_cycle;
  }

  static bool Read(::views_bridge_mac::mojom::BridgedNativeWidgetInitParams::DataView input, ::views_bridge_mac::mojom::BridgedNativeWidgetInitParamsPtr* output);
};

}  // namespace mojo

#endif  // UI_VIEWS_BRIDGE_MAC_MOJO_BRIDGED_NATIVE_WIDGET_MOJOM_H_